from __future__ import unicode_literals
from future.builtins import str
import re
from django import forms
from django.core import validators
from django.core.validators import EmailValidator
from django.utils.translation import gettext_lazy


class EmailListField(forms.Field):
    """
    A form field that parses and validates a list of email addresses.

    The input can be a list, tuple or string. If it is a string,
    we split the email addresses by whitespace or comma.
    """
    default_error_messages = {
        'invalid': gettext_lazy('Type email-addresses separated by newlines, whitespace or comma.')
    }
    default_help_text = gettext_lazy('Type email-addresses separated by newlines, whitespace or comma.')
    widget = forms.Textarea
    stringsplit_regex = re.compile(r'(?:\s*,\s*)|\s+')
    email_validator = EmailValidator()

    def __init__(self, *args, **kwargs):
        """
        Works just like any other form field.

        Parameters are just like for any other form field, with the
        addition of the ``extra_help_text`` parameter. The ``extra_help_text``
        parameter can be used to append help text to the default help text.
        """
        if 'help_text' not in kwargs:
            kwargs['help_text'] = self.default_help_text
        extra_help_text = kwargs.pop('extra_help_text', None)
        if extra_help_text:
            kwargs['help_text'] = u'{} {}'.format(kwargs['help_text'], extra_help_text)
        super(EmailListField, self).__init__(*args, **kwargs)

    def string_to_list(self, value):
        value = value.strip()
        if value.strip():
            return self.stringsplit_regex.split(value)
        else:
            return []

    def to_python_list(self, value):
        if value in validators.EMPTY_VALUES:
            return []
        elif isinstance(value, list):
            return value
        elif isinstance(value, tuple):
            return list(value)
        elif isinstance(value, str):
            return self.string_to_list(value)
        else:
            raise TypeError('Invalid type for EmailListField: {}'.format(type(value)))

    def validate_list_of_emails(self, list_of_emails):
        for email in list_of_emails:
            try:
                self.email_validator(email)
            except forms.ValidationError:
                errormessage = gettext_lazy('Invalid email address: %(email)s') % {'email': email}
                raise forms.ValidationError(errormessage)

    def to_python(self, value):
        list_of_emails = self.to_python_list(value)
        self.validate_list_of_emails(list_of_emails)
        return list_of_emails



from __future__ import unicode_literals
from collections import OrderedDict
import urllib
from cradmin_legacy.viewhelpers.listfilter.base.exceptions import InvalidFiltersStringError
from future import standard_library

standard_library.install_aliases()


class FiltersHandler(object):
    """
    Parser of the ``filters_string``. See :meth:`.AbstractFilterList.parse_filters_string`.
    """

    #: The string separating filters in the filters string. Defaults to ``"/"``.
    filter_separator = '/'

    #: The string used to separate filter slug and value.
    #: It does not matter if this also appears in the value,
    #: we handle that by splitting with a maxsplit of 1.
    #:
    #: Defaults to ``"-"``.
    slug_and_value_separator = '-'

    #: The string used to separate multivalue strings.
    #: This string can not appear in any value used by the filter.
    #:
    #: Defaults to ``","``.
    multivalue_separator = ','

    def __init__(self, urlbuilder):
        """
        Parameters:
            urlbuilder: A method that takes a single argument, ``filters_string``,
                and returns an absolute URL with that string injected as the
                filters string. The filters_string is urlencoded, so you should
                be able to just forward it to ``reverse`` for your view.
        """
        self.filtermap = OrderedDict()
        self.urlbuilder = urlbuilder
        self._parse_called = False

    def split_raw_filter_values(self, raw_values):
        """
        Parse the given ``value``, splitting it into a list of values.

        You can override this if just overriding :obj:`.FiltersHandler.multivalue_separator`
        is not powerful enough.

        If you override this, you will also have ot override :meth:`.join_filter_values`.
        """
        values = urllib.parse.unquote_plus(raw_values)
        return [urllib.parse.unquote_plus(value)
                for value in values.split(self.multivalue_separator)]

    def join_filter_values(self, values):
        """
        The reverse of :meth:`.split_raw_filter_values`. Joins
        the given ``values`` list into a string.
        """
        # We quote each value, and then we quote the entire string. This
        # ensures that we do not get any problems when a value contains
        # ``multivalue_separator``.
        raw_values = [urllib.parse.quote_plus(value) for value in values]
        raw_values = self.multivalue_separator.join(raw_values)
        return urllib.parse.quote_plus(raw_values)

    def parse_filter_string(self, filter_string):
        """
        Parse the given ``filter_string`` and return a ``(slug, values)`` tuple,
        where ``slug`` is a filter slug and ``values`` is a list of strings.

        You should not need to override this.
        """
        if self.slug_and_value_separator not in filter_string:
            raise InvalidFiltersStringError('"{}" does not contain "{}".'.format(
                filter_string, self.slug_and_value_separator))
        slug, value = filter_string.split(self.slug_and_value_separator, 1)
        return slug, self.split_raw_filter_values(value)

    def parse(self, filters_string):
        """
        Parse the given ``filters_string`` and add any values
        found in the string to the corresponding filter.

        You should not need to override this.
        """
        if self._parse_called:
            raise RuntimeError('Can not call parse multiple times on a FiltersHandler.')
        self._parse_called = True

        if not filters_string:
            return

        filters_string = filters_string.strip(self.filter_separator)
        for filter_string in filters_string.split(self.filter_separator):
            slug, values = self.parse_filter_string(filter_string)
            if slug not in self.filtermap:
                raise InvalidFiltersStringError('"{}" is not a valid filter slug.'.format(slug))
            self.filtermap[slug].set_values(values)

    def add_filter(self, filterobject):
        """
        Add a :class:`.AbstractFilter` to the handler.
        """
        slug = filterobject.get_slug()
        if slug in self.filtermap:
            raise ValueError('Duplicate slug: "{}".'.format(slug))
        if self.slug_and_value_separator in slug:
            raise ValueError('Invalid filter slug: "{}". Slugs can not contain "{}".'.format(
                slug, self.slug_and_value_separator))
        self.filtermap[slug] = filterobject

    def normalize_values(self, values):
        """
        Normalize values list to only contain
        ``bool(value) == True`` values. Since values
        is a list of strings, this means that it strips
        out all empty strings.
        """
        return [value for value in values if value]

    def build_filter_string(self, slug, values):
        """
        Build a filter string suitable for an URL from the given
        ``slug`` and ``values``.

        Parameters:
            slug: See :meth:`.AbstractFilter.get_slug`.
            value: A list of values. All items in the list must be strings.
        """
        return '{slug}{separator}{values}'.format(
            slug=slug,
            separator=self.slug_and_value_separator,
            values=self.join_filter_values(values=values))

    def build_filters_string(self, changed_filterobject):
        """
        Build the ``filters_string`` for :meth:`.build_filter_url`.
        """
        filters_strings = []
        for slug, filterobject in self.filtermap.items():
            if filterobject.get_slug() == changed_filterobject.get_slug():
                values = changed_filterobject.values
            else:
                values = filterobject.values
            values = self.normalize_values(values)
            if values:
                filters_strings.append(self.build_filter_string(slug=slug, values=values))
        return self.filter_separator.join(filters_strings)

    def build_filter_url(self, changed_filterobject):
        """
        Build an URL that applies the change introduced by
        ``changed_filterobject`` while keeping any values
        in all the other filters within the handler.

        Parameters:
            changed_filterobject: A :class:`.AbstractFilter` object.
        """
        filters_string = self.build_filters_string(changed_filterobject=changed_filterobject)
        return self.urlbuilder(filters_string=filters_string)

    def filter(self, queryobject, exclude=None):
        """
        Apply the filters to the given ``queryobject``.

        Loops through all the registered filters (the filters added with :meth:`.add_filter`),
        and run :meth:`.AbstractFilter.filter`.

        Parameters:
            queryobject: See :meth:`.AbstractFilter.filter`.
            exclude: Set with the slugs of filters to ignore/exclude when filtering.
                Defaults to ``None``, which means that all filters are applied.
        """
        for filterobject in self.filtermap.values():
            if exclude and filterobject.get_slug() in exclude:
                continue
            queryobject = filterobject.filter(queryobject=queryobject)
        return queryobject

    def get_label_for(self, slug):
        """
        Get the label for the filter registered in the filterhandler with the given ``slug``.

        Raises:
            KeyError: If the ``slug`` is not registered.
        """
        return self.filtermap[slug].get_label()

    def get_cleaned_value_for(self, slug):
        """
        Get the cleaned value for the filter registered in the filterhandler
        with the given ``slug``.

        Raises:
            KeyError: If the ``slug`` is not registered.
        """
        return self.filtermap[slug].get_cleaned_value()

    def get_cleaned_values_for(self, slug):
        """
        Get the cleaned values (list) for the filter registered in the filterhandler
        with the given ``slug``.

        Raises:
            KeyError: If the ``slug`` is not registered.
        """
        return self.filtermap[slug].get_cleaned_values()



from __future__ import unicode_literals
import json
from django import forms
from django.template.loader import render_to_string


class AceMarkdownWidget(forms.widgets.Textarea):
    template_name = 'cradmin_legacy/acemarkdown/widget.django.html'
    directiveconfig = {
        # 'showTextarea': False,
        # 'theme': 'tomorrow'
    }

    @property
    def media(self):
        return forms.Media(
            js=[
                'cradmin_legacy/dist/vendor/js/ace-editor/ace.js',
            ]
        )

    def get_context(self, name, value, attrs):
        attrs = attrs.copy()
        attrs['textarea cradmin-legacy-acemarkdown-textarea'] = ''
        if 'required' in attrs:
            attrs['required'] = False
        return {
            'directiveconfig': json.dumps(self.directiveconfig),
            'widget': {
                'name': name,
                'value': value,
                'attrs': attrs
            }
        }



from __future__ import unicode_literals
from django.views.generic.detail import DetailView as DjangoDetailView
from cradmin_legacy.viewhelpers.mixins import QuerysetForRoleMixin


class DetailView(QuerysetForRoleMixin, DjangoDetailView):
    """
    Base detail view for Django cradmin views.
    """


class DetailRoleView(DetailView):
    """
    Extends :class:`.DetailView` to streamline creating a detail view
    for the current role object.

    Just like :class:`.DetailView`, but with the get_object and
    get_queryset_for_role methods implemented to edit the current role
    object.
    """
    def get_object(self, queryset=None):
        return self.get_queryset_for_role(self.request.cradmin_role).get()

    def get_queryset_for_role(self, role):
        return self.model.objects.filter(pk=role.pk)



from . import mixins  # noqa
from . import crudbase  # noqa
from . import create  # noqa
from . import update  # noqa
from . import delete  # noqa
from . import detail  # noqa
from . import formbase  # noqa
from . import listbuilderview  # noqa
from . import multiselect  # noqa
from . import objecttable  # noqa

__all__ = [
    'mixins',
    'crudbase',
    'create',
    'update',
    'delete',
    'detail',
    'formbase',
    'listbuilderview',
    'multiselect',
    'objecttable',
]



from __future__ import unicode_literals
from django import template
import logging

from django.conf import settings

from django.template.defaultfilters import stringfilter

from cradmin_legacy import css_icon_map

register = template.Library()
log = logging.getLogger(__name__)


@register.simple_tag
@stringfilter
def cradmin_icon(iconkey):
    """
    Returns the css class for an icon configured with the
    given key in ``CRADMIN_LEGACY_CSS_ICON_MAP``.
    """
    iconmap = getattr(settings, 'CRADMIN_LEGACY_CSS_ICON_MAP', css_icon_map.FONT_AWESOME)
    icon_classes = iconmap.get(iconkey, '')
    if not icon_classes:
        log.warn('No icon named "%s" in settings.CRADMIN_LEGACY_ICONMAP.', iconkey)
    return icon_classes



from __future__ import unicode_literals
from django.contrib.auth import get_user_model


def create_user(username=None, password='test', **kwargs):
    """
    Create a user with the given ``username``.

    .. deprecated:: 1.0.1b8
        Use model_bakery instead.
    """
    user = get_user_model().objects.create(username=username, **kwargs)
    user.set_password(password)
    user.save()
    return user



from __future__ import unicode_literals
from builtins import object


class QuerysetForRoleMixin(object):
    def get_queryset_for_role(self, role):
        """
        Get a queryset with all objects of ``self.model``  that
        the current role can access.
        """
        raise NotImplementedError()

    def get_queryset(self):
        """
        DO NOT override this. Override :meth:`.get_queryset_for_role`
        instead.
        """
        queryset = self.get_queryset_for_role(self.request.cradmin_role)
        return queryset



from django.test import TestCase
import htmls

from cradmin_legacy.renderable import AbstractRenderableWithCss
from cradmin_legacy.viewhelpers import listbuilder
from cradmin_legacy.python2_compatibility import mock


class MinimalRenderable(AbstractRenderableWithCss):
    template_name = 'cradmin_viewhelpers_testapp/listbuilder/minimal-renderable.django.html'


class TestAbstractItemRenderer(TestCase):
    def test_without_valuealias(self):
        itemrenderer = listbuilder.base.AbstractItemRenderer(value='testvalue')
        self.assertEqual('testvalue', itemrenderer.value)

    def test_with_valuealias(self):
        class MyItemRenderer(listbuilder.base.AbstractItemRenderer):
            valuealias = 'myvalue'

        itemrenderer = MyItemRenderer(value='testvalue')
        self.assertEqual('testvalue', itemrenderer.value)
        self.assertEqual('testvalue', itemrenderer.myvalue)


class TestItemValueRenderer(TestCase):
    def test_render(self):
        rendered = listbuilder.base.ItemValueRenderer(value='testvalue').render()
        selector = htmls.S(rendered)
        self.assertEqual('testvalue',
                         selector.one('.cradmin-legacy-listbuilder-itemvalue').alltext_normalized)


class TestItemFrameRenderer(TestCase):
    def test_render(self):
        rendered = listbuilder.base.ItemFrameRenderer(
            inneritem=listbuilder.base.ItemValueRenderer(value='testvalue')).render(request=mock.MagicMock())
        selector = htmls.S(rendered)
        self.assertTrue(selector.exists('.cradmin-legacy-listbuilder-itemframe'))
        self.assertEqual('testvalue',
                         selector.one('.cradmin-legacy-listbuilder-itemvalue').alltext_normalized)


class TestList(TestCase):
    def test_append(self):
        testlist = listbuilder.base.List()
        renderable = MinimalRenderable()
        testlist.append(renderable)
        self.assertEqual([renderable], testlist.renderable_list)

    def test_extend(self):
        testlist = listbuilder.base.List()
        renderable1 = MinimalRenderable()
        renderable2 = MinimalRenderable()
        testlist.extend([renderable1, renderable2])
        self.assertEqual([renderable1, renderable2], testlist.renderable_list)

    def test_extend_with_values_without_frame_renderer_class(self):
        testlist = listbuilder.base.List()
        testlist.extend_with_values(['testvalue1', 'testvalue2'],
                                    value_renderer_class=listbuilder.base.ItemValueRenderer)
        self.assertEqual('testvalue1', testlist.renderable_list[0].value)
        self.assertEqual('testvalue2', testlist.renderable_list[1].value)
        self.assertTrue(isinstance(testlist.renderable_list[0], listbuilder.base.ItemValueRenderer))
        self.assertTrue(isinstance(testlist.renderable_list[1], listbuilder.base.ItemValueRenderer))

    def test_extend_with_values_with_frame_renderer_class(self):
        testlist = listbuilder.base.List()
        testlist.extend_with_values(['testvalue1', 'testvalue2'],
                                    value_renderer_class=listbuilder.base.ItemValueRenderer,
                                    frame_renderer_class=listbuilder.base.ItemFrameRenderer)
        self.assertEqual('testvalue1', testlist.renderable_list[0].value)
        self.assertEqual('testvalue2', testlist.renderable_list[1].value)
        self.assertTrue(isinstance(testlist.renderable_list[0], listbuilder.base.ItemFrameRenderer))
        self.assertTrue(isinstance(testlist.renderable_list[1], listbuilder.base.ItemFrameRenderer))

    def test_extend_with_values_value_and_frame_renderer_kwargs_dict(self):
        testlist = listbuilder.base.List()
        testlist.extend_with_values(['testvalue1', 'testvalue2'],
                                    value_renderer_class=listbuilder.base.ItemValueRenderer,
                                    value_and_frame_renderer_kwargs={'extra_kwarg': 10})
        self.assertEqual({'extra_kwarg': 10}, testlist.renderable_list[0].kwargs)
        self.assertEqual({'extra_kwarg': 10}, testlist.renderable_list[1].kwargs)

    def test_extend_with_values_value_and_frame_renderer_kwargs_function(self):
        def kwargs_function(value):
            if value == 'testvalue1':
                return {'is_first': True}
            else:
                return {}

        testlist = listbuilder.base.List()
        testlist.extend_with_values(['testvalue1', 'testvalue2'],
                                    value_renderer_class=listbuilder.base.ItemValueRenderer,
                                    value_and_frame_renderer_kwargs=kwargs_function)
        self.assertEqual({'is_first': True}, testlist.renderable_list[0].kwargs)
        self.assertEqual({}, testlist.renderable_list[1].kwargs)

    def test_from_value_iterable_without_frame_renderer_class(self):
        testlist = listbuilder.base.List.from_value_iterable(
            ['testvalue1', 'testvalue2'],
            value_renderer_class=listbuilder.base.ItemValueRenderer)
        self.assertEqual('testvalue1', testlist.renderable_list[0].value)
        self.assertEqual('testvalue2', testlist.renderable_list[1].value)
        self.assertTrue(isinstance(testlist.renderable_list[0], listbuilder.base.ItemValueRenderer))
        self.assertTrue(isinstance(testlist.renderable_list[1], listbuilder.base.ItemValueRenderer))

    def test_from_value_iterable_with_frame_renderer_class(self):
        testlist = listbuilder.base.List.from_value_iterable(
            ['testvalue1', 'testvalue2'],
            value_renderer_class=listbuilder.base.ItemValueRenderer,
            frame_renderer_class=listbuilder.base.ItemFrameRenderer)
        self.assertEqual('testvalue1', testlist.renderable_list[0].value)
        self.assertEqual('testvalue2', testlist.renderable_list[1].value)
        self.assertTrue(isinstance(testlist.renderable_list[0], listbuilder.base.ItemFrameRenderer))
        self.assertTrue(isinstance(testlist.renderable_list[1], listbuilder.base.ItemFrameRenderer))

    def test_render(self):
        testlist = listbuilder.base.List()
        testlist.append(listbuilder.base.ItemValueRenderer(value='testvalue1'))
        testlist.append(listbuilder.base.ItemValueRenderer(value='testvalue2'))
        selector = htmls.S(testlist.render(request=mock.MagicMock()))
        self.assertEqual(2, selector.count('li'))
        self.assertEqual('testvalue1', selector.list('li')[0].alltext_normalized)
        self.assertEqual('testvalue2', selector.list('li')[1].alltext_normalized)



from cradmin_legacy import crapp
from cradmin_legacy.superuserui.views import createview
from cradmin_legacy.superuserui.views import deleteview
from cradmin_legacy.superuserui.views import editview
from cradmin_legacy.superuserui.views import listview


class DjangoModelCrApp(crapp.App):
    """
    Cradmin App that provides list, create, update and delete views
    for a Django model.
    """

    #: The :class:`cradmin_legacy.superuserui.superuserui_registry.ModelConfig` this
    #: app belongs to.
    #: This is set automatically in
    #: :meth:`cradmin_legacy.superuserui.superuserui_registry.ModelConfig.make_crapp_class`
    modelconfig = None

    @classmethod
    def get_listview_class(cls):
        return listview.View

    @classmethod
    def get_foreignkeyselectview_class(cls):
        return listview.ForeignKeySelectView

    @classmethod
    def get_manytomanyselectview_class(cls):
        return listview.ManyToManySelectView

    @classmethod
    def get_editview_class(cls):
        return editview.View

    @classmethod
    def get_createview_class(cls):
        return createview.View

    @classmethod
    def get_deleteview_class(cls):
        return deleteview.View

    @classmethod
    def add_listview(cls, appurls):
        listview_class = cls.get_listview_class()
        if listview_class:
            appurls.append(crapp.Url(
                r'^$',
                listview_class.as_view(),
                name=crapp.INDEXVIEW_NAME))
            appurls.append(crapp.Url(
                r'^filter/(?P<filters_string>.+)?$',
                listview_class.as_view(),
                name='filter'))
        else:
            raise NotImplementedError('You must return a view class from get_listview_class(). '
                                      'Note that it can be something other than a list view, '
                                      'such as an overview of with information/stats for the model.')

    @classmethod
    def add_foreignkeyselectview(cls, appurls):
        foreignkeyselectview_class = cls.get_foreignkeyselectview_class()
        if foreignkeyselectview_class:
            appurls.append(crapp.Url(
                r'^foreignkeyselect$',
                foreignkeyselectview_class.as_view(),
                name='foreignkeyselect'))
            appurls.append(crapp.Url(
                r'^foreignkeyselect/filter/(?P<filters_string>.+)?$',
                foreignkeyselectview_class.as_view(),
                name='foreignkeyselect-filter'))

    @classmethod
    def add_manytomanyselectview(cls, appurls):
        manytomanyselectview_class = cls.get_manytomanyselectview_class()
        if manytomanyselectview_class:
            appurls.append(crapp.Url(
                r'^manytomanyselect$',
                manytomanyselectview_class.as_view(),
                name='manytomanyselect'))
            appurls.append(crapp.Url(
                r'^manytomanyselect/filter/(?P<filters_string>.+)?$',
                manytomanyselectview_class.as_view(),
                name='manytomanyselect-filter'))

    @classmethod
    def add_createview(cls, appurls):
        createview_class = cls.get_createview_class()
        if createview_class:
            appurls.append(crapp.Url(
                r'^create/$',
                createview_class.as_view(),
                name="create"))

    @classmethod
    def add_editview(cls, appurls):
        editview_class = cls.get_editview_class()
        if editview_class:
            appurls.append(crapp.Url(
                r'^edit/(?P<pk>\d+)$',
                editview_class.as_view(),
                name="edit"))

    @classmethod
    def add_deleteview(cls, appurls):
        deleteview_class = cls.get_deleteview_class()
        if deleteview_class:
            appurls.append(crapp.Url(
                r'^delete/(?P<pk>\d+)$',
                deleteview_class.as_view(),
                name="delete"))

    @classmethod
    def get_appurls(cls):
        appurls = []
        cls.add_listview(appurls=appurls)
        cls.add_foreignkeyselectview(appurls=appurls)
        cls.add_manytomanyselectview(appurls=appurls)
        cls.add_createview(appurls=appurls)
        cls.add_editview(appurls=appurls)
        cls.add_deleteview(appurls=appurls)

        return appurls



from __future__ import unicode_literals
import json
from builtins import str
from datetime import timedelta, datetime

from django.forms import widgets
from django.template import loader
from django.utils import timezone
from django.utils.translation import gettext_lazy
from django.utils.translation import pgettext_lazy

from cradmin_legacy.templatetags.cradmin_legacy_icon_tags import cradmin_icon
from cradmin_legacy.widgets.selectwidgets import WrappedSelect


class DatePickerWidget(widgets.TextInput):
    """
    A Widget for selecting a date.

    This implements a Django widget using ``cradmin-legacy-datetime-selector``
    AngularJS directive, but defaults to values that makes the directive
    only allows the user to pick a date.

    The widget works for both date and datetime fields. Datetime fields
    will just get the time, minutes and seconds set to ``0``.

    Also serves as the base class for :class:`.BetterDateTimePickerWidget`.

    You can configure almost everything by extending the class,
    or (to a lesser extent) via parameters to the contructor.

    The AngularJS directive uses momentjs.
    MomentJS has their own i18n support which we use to format date and time.
    You can override the locale/language used by momentjs via the
    ``CRADMIN_LEGACY_MOMENTJS_LOCALE`` Django setting,
    and Django-cradmin contains all the locale files bundled with MomentJS.

    You can easily provide your own translation files as explained in the momentjs docs.
    If your template inherits from any of the Django-cradmin templates (which
    all extend from ``cradmin_legacy/standalone-base-internal.django.html``, you
    add your momentjs locale by overriding the the ``momentjslocale`` templatate block.
    Remember that this can be overridden for your entire project by providing a
    ``cradmin_legacy/standalone-base.django.html`` that overrides the ``momentjslocale``
    block.
    """

    #: The template used to render the widget.
    template_name = 'cradmin_legacy/widgets/datetimepicker.django.html'

    #: The momentjs formatting string to use to format the value
    #: of the hidden field where the actually posted value is
    #: set.
    #:
    #: You normally do not need to override this, since ISO format
    #: should work for all use cases, and the user will never see
    #: this value (it is stored in a hidden field).
    #:
    #: See the docs for the ``format()`` function in the
    #: `momentjs docs <http://momentjs.com/>`_ for the possible
    #: formatting strings.
    destinationfield_momentjs_format = 'YYYY-MM-DD'

    #: Is the field required?
    #:
    #: Can be overridden via the ``required``
    #: keyword argument for ``__init__``, and should
    #: be set to ``True`` for required fields.
    default_required = False

    #: The label of the button used to trigger the date selector
    #: when the field has a value.
    #:
    #: Can be overridden via the ``buttonlabel``
    #: keyword argument for ``__init__``.
    default_buttonlabel = pgettext_lazy('Has value trigger button text for date widget', 'Change date')

    #: The label of the button used to trigger the date selector
    #: when the field has no value.
    #:
    #: Can be overridden via the ``buttonlabel_novalue``
    #: keyword argument for ``__init__``.
    default_buttonlabel_novalue = pgettext_lazy('No value trigger button text for date widget', 'Select a date')

    #: The label for the _Use_ buttons (both in the mobile view and on the
    #: time picker page on desktop/table).
    #:
    #: Can be overridden via the ``usebuttonlabel``
    #: keyword argument for ``__init__``.
    default_usebuttonlabel = pgettext_lazy('Use button text in datetime widget', 'Use')

    #: The iconkey for :func:`.cradmin_legacy.templatetags.cradmin_legacy_icon_tags`
    #: to use for the close button (the X in the top right corner).
    #:
    #: Can be overridden via the ``back_close_iconkey``
    #: keyword argument for ``__init__``.
    default_close_iconkey = 'x'

    #: The iconkey for :func:`.cradmin_legacy.templatetags.cradmin_legacy_icon_tags`
    #: to use for the back arrow button for navigating from the time select
    #: page back to the date picker page.
    #:
    #: Can be overridden via the ``back_iconkey``
    #: keyword argument for ``__init__``.
    default_back_iconkey = 'close-overlay-right-to-left'

    #: The momentjs formatting string to use to format the preview of the
    #: selected date on the second page of the
    #:
    #: See the docs for the ``format()`` function in the
    #: `momentjs docs <http://momentjs.com/>`_ for the possible
    #: formatting strings.
    #:
    #: Can be overridden via the ``timeselector_datepreview_momentjs_format``
    #: keyword argument for ``__init__``.
    default_timeselector_datepreview_momentjs_format = 'LL'

    #: The text shown in the date picker table to indicate the selected date.
    #: You can set this to an empty string if you do not want to show this
    #: label.
    #:
    #: Can be overridden via the ``selected_day_label_text``
    #: keyword argument for ``__init__``.
    default_selected_day_label_text = pgettext_lazy('Selected label in table cell in datetime widget', 'selected')

    #: The text shown in the date picker table cell for "Today".
    #: You can set this to an empty string if you do not want to show this
    #: label.
    #:
    #: Can be overridden via the ``today_label_text``
    #: keyword argument for ``__init__``.
    default_today_label_text = pgettext_lazy('Today label in table cell in datetime widget', 'today')

    #: See :meth:`~.DatePickerWidget.get_preview_angularjs_template`.
    #:
    #: Can be overridden via the ``preview_angularjs_template``
    #: keyword argument for ``__init__``.
    default_preview_angularjs_template = "{{ momentObject.format('LL') }}"

    #: The aria-label (screenreader only label) for the close button.
    #:
    #: Can be overridden via the ``close_screenreader_text``
    #: keyword argument for ``__init__``.
    default_close_screenreader_text = pgettext_lazy('Datetime widget screenreader',
                                                    'Close date picker without changing the value')

    #: The screenreader only label for the select year ``<select>``.
    #:
    #: Can be overridden via the ``year_screenreader_text``
    #: keyword argument for ``__init__``.
    default_year_screenreader_text = pgettext_lazy('Datetime widget screenreader', 'Select year')

    #: The screenreader only label for the select month ``<select>``.
    #:
    #: Can be overridden via the ``month_screenreader_text``
    #: keyword argument for ``__init__``.
    default_month_screenreader_text = pgettext_lazy('Datetime widget screenreader', 'Select month')

    #: The screenreader only label for the select day ``<select>``.
    #:
    #: Can be overridden via the ``day_screenreader_text``
    #: keyword argument for ``__init__``.
    default_day_screenreader_text = pgettext_lazy('Datetime widget screenreader', 'Select day')

    #: The screenreader only label for the select hour ``<select>``.
    #:
    #: Can be overridden via the ``hour_screenreader_text``
    #: keyword argument for ``__init__``.
    default_hour_screenreader_text = pgettext_lazy('Datetime widget screenreader', 'Select hour')

    #: The screenreader only label for the select minute ``<select>``.
    #:
    #: Can be overridden via the ``minute_screenreader_text``
    #: keyword argument for ``__init__``.
    default_minute_screenreader_text = pgettext_lazy('Datetime widget screenreader', 'Select minute')

    #: The screenreader only prefix for the arial-label of the "Use"-button
    #: The suffix is the formatted value of the selected date, formatted using
    #: :obj:`~.DatePickerWidget.default_usebutton_arialabel_momentjs_format`.
    #:
    #: Can be overridden via the ``usebutton_arialabel_prefix``
    #: keyword argument for ``__init__``.
    default_usebutton_arialabel_prefix = pgettext_lazy('Datetime widget screenreader',
                                                       'Confirm that you want to select')

    #: The momentjs format for the screenreader only suffox for the arial-label
    #: of the "Use"-button. The prefix is configured in
    #: :obj:`~.DatePickerWidget.default_usebutton_arialabel_prefix`.
    #:
    #: Can be overridden via the ``usebutton_arialabel_momentjs_format``
    #: keyword argument for ``__init__``.
    default_usebutton_arialabel_momentjs_format = 'LL'

    #: The screenreader only aria-label for the button that takes the user
    #: back to the datepicker when they are in the time picker (not used on mobile).
    #:
    #: Can be overridden via the ``back_to_datepicker_screenreader_text``
    #: keyword argument for ``__init__``.
    default_back_to_datepicker_screenreader_text = gettext_lazy('Return to date picker')

    #: The screenreader only date-picker table caption.
    #:
    #: Can be overridden via the ``dateselector_table_screenreader_caption``
    #: keyword argument for ``__init__``.
    default_dateselector_table_screenreader_caption = pgettext_lazy(
        'Datetime widget screenreader',
        'Select date. Navigate with the arrow keys or tab, '
        'jump up to the month selector with the page up key '
        'and back to this table with the page down key.')

    #: Default button text for the now button.
    #:
    #: Can be overridden via the ``now_button_text``
    #: keyword argument for ``__init__``.
    default_now_button_text = pgettext_lazy('Now button text datetime widget', 'Now')

    #: Default button text for the today button.
    #:
    #: Can be overridden via the ``today_button_text``
    #: keyword argument for ``__init__``.
    default_today_button_text = pgettext_lazy('Today button text datetime widget', 'Today')

    #: Default button text for the clear button.
    #:
    #: Can be overridden via the ``clear_button_text``
    #: keyword argument for ``__init__``.
    default_clear_button_text = pgettext_lazy('Clear button text datetime widget', 'Clear')

    #: Default label to display before the date ``<select>`` boxes.
    #: If this is set to empty string (the default), the label is not shown.
    #:
    #: Shown in the mobile datetime picker, but not in the date picker for
    #: desktop/tablet (since it would be strange to show ``date: <month> <year>``).
    #:
    #: Can be overridden via the ``date_label_text``
    #: keyword argument for ``__init__``.
    default_date_label_text = ''

    #: Default label to display before the time ``<select>`` boxes.
    #: You can set this to empty string to disable the label.
    #:
    #: Shown in the mobile datetime picker, and in the time picker for
    #: desktop/tablet.
    #:
    #: Can be overridden via the ``time_label_text``
    #: keyword argument for ``__init__``.
    default_time_label_text = pgettext_lazy('Time label datetime widget', 'Time')

    def __init__(self, *args, **kwargs):
        """
        Parameters:
            required: See :obj:`.DatePickerWidget.default_required`.
            buttonlabel: See :obj:`.DatePickerWidget.default_buttonlabel`.
            buttonlabel_novalue: See :obj:`.DatePickerWidget.default_buttonlabel_novalue`.
            usebuttonlabel: See :obj:`.DatePickerWidget.default_usebuttonlabel`.
            close_iconkey: See :obj:`.DatePickerWidget.default_close_iconkey`.
            back_iconkey: See :obj:`.DatePickerWidget.default_back_iconkey`.
            today_label_text: See :obj:`.DatePickerWidget.default_today_label_text`.
            date_label_text: See :obj:`.DatePickerWidget.default_date_label_text`.
            time_label_text: See :obj:`.DatePickerWidget.default_time_label_text`.
            selected_day_label_text: See :obj:`.DatePickerWidget.default_selected_day_label_text`.
            timeselector_datepreview_momentjs_format: See
                :obj:`.DatePickerWidget.default_timeselector_datepreview_momentjs_format`.
            no_value_preview_text: See :obj:`.DatePickerWidget.default_no_value_preview_text`.
            preview_angularjs_template: See :obj:`.DatePickerWidget.default_preview_angularjs_template`.
            close_screenreader_text: See :obj:`.DatePickerWidget.default_close_screenreader_text`.
            year_screenreader_text: See :obj:`.DatePickerWidget.default_year_screenreader_text`.
            month_screenreader_text: See :obj:`.DatePickerWidget.default_month_screenreader_text`.
            day_screenreader_text: See :obj:`.DatePickerWidget.default_day_screenreader_text`.
            hour_screenreader_text: See :obj:`.DatePickerWidget.default_hour_screenreader_text`.
            minute_screenreader_text: See :obj:`.DatePickerWidget.default_minute_screenreader_text`.
            clear_button_text: See :obj:`.DatePickerWidget.default_clear_button_text`.
            now_button_text: See :obj:`.DatePickerWidget.default_now_button_text`.
            today_button_text: See :obj:`.DatePickerWidget.default_today_button_text`.
            usebutton_arialabel_prefix: See :obj:`.DatePickerWidget.default_usebutton_arialabel_prefix`.
            usebutton_arialabel_momentjs_format: See
                :obj:`.DatePickerWidget.default_usebutton_arialabel_momentjs_format`.
            back_to_datepicker_screenreader_text: See
                :obj:`.DatePickerWidget.default_back_to_datepicker_screenreader_text`.
            dateselector_table_screenreader_caption: See
                :obj:`.DatePickerWidget.default_dateselector_table_screenreader_caption`.
            minimum_datetime: The minimum datetime allowed to be select in the widget.
            maximum_datetime: The minimum datetime allowed to be select in the widget.
            default_now_time: Dictionary of default time for now. A dictionary that maps hours and 
                minutes to for the default now date. Format: {'hour': <int>, 'minute': <int>}
        """
        self.required = kwargs.pop('required', self.default_required)
        self.buttonlabel = kwargs.pop('buttonlabel', self.default_buttonlabel)
        self.buttonlabel_novalue = kwargs.pop('buttonlabel_novalue', self.default_buttonlabel_novalue)
        self.today_label_text = kwargs.pop('today_label_text', self.default_today_label_text)
        self.selected_day_label_text = kwargs.pop('selected_day_label_text', self.default_selected_day_label_text)
        self.usebuttonlabel = kwargs.pop('usebuttonlabel', self.default_usebuttonlabel)
        self.close_iconkey = kwargs.pop('close_iconkey', self.default_close_iconkey)
        self.back_iconkey = kwargs.pop('back_iconkey', self.default_back_iconkey)
        self.timeselector_datepreview_momentjs_format = kwargs.pop(
            'timeselector_datepreview_momentjs_format',
            self.default_timeselector_datepreview_momentjs_format)
        self.no_value_preview_text = kwargs.pop('no_value_preview_text', '')
        self.preview_angularjs_template = kwargs.pop('preview_angularjs_template',
                                                     self.default_preview_angularjs_template)

        self.close_screenreader_text = kwargs.pop('close_screenreader_text',
                                                  self.default_close_screenreader_text)
        self.year_screenreader_text = kwargs.pop('year_screenreader_text',
                                                 self.default_year_screenreader_text)
        self.month_screenreader_text = kwargs.pop('month_screenreader_text',
                                                  self.default_month_screenreader_text)
        self.day_screenreader_text = kwargs.pop('day_screenreader_text',
                                                self.default_day_screenreader_text)
        self.hour_screenreader_text = kwargs.pop('hour_screenreader_text',
                                                 self.default_hour_screenreader_text)
        self.minute_screenreader_text = kwargs.pop('minute_screenreader_text',
                                                   self.default_minute_screenreader_text)
        self.usebutton_arialabel_prefix = kwargs.pop('usebutton_arialabel_prefix',
                                                     self.default_usebutton_arialabel_prefix)
        self.usebutton_arialabel_momentjs_format = kwargs.pop('usebutton_arialabel_momentjs_format',
                                                              self.default_usebutton_arialabel_momentjs_format)
        self.back_to_datepicker_screenreader_text = kwargs.pop('back_to_datepicker_screenreader_text',
                                                               self.default_back_to_datepicker_screenreader_text)
        self.dateselector_table_screenreader_caption = kwargs.pop('dateselector_table_screenreader_caption',
                                                                  self.default_dateselector_table_screenreader_caption)
        self.minimum_datetime = kwargs.pop('minimum_datetime', None)
        self.maximum_datetime = kwargs.pop('maximum_datetime', None)
        self.now_button_text = kwargs.pop('now_button_text', self.default_now_button_text)
        self.today_button_text = kwargs.pop('today_button_text', self.default_today_button_text)
        self.clear_button_text = kwargs.pop('clear_button_text', self.default_clear_button_text)
        self.date_label_text = kwargs.pop('date_label_text', self.default_date_label_text)
        self.time_label_text = kwargs.pop('time_label_text', self.default_time_label_text)
        self.default_now_time = kwargs.pop('default_now_time', None)

        super(DatePickerWidget, self).__init__(*args, **kwargs)

    def get_datepicker_config(self, fieldid, triggerbuttonid, previewid,
                              previewtemplateid):
        """
        Get the configuration for the ``cradmin-legacy-datetime-selector`` AngularJS directive.

        This is encoded as json, so all values must be JSON-encodable.
        This means that you need to ensure any lazy translation string is
        wrapped with ``str`` (or ``unicode`` for python2).

        You should normally not need to override this, since everything is
        configurable via ``__init__`` kwargs or class attributes.
        """
        now = timezone.now()
        if timezone.is_aware(now):
            now = timezone.localtime(now)
        configdict = {
            'now': now.isoformat(),
            'destinationfieldid': fieldid,
            'previewid': previewid,
            'previewtemplateid': previewtemplateid,
            'triggerbuttonid': triggerbuttonid,
            'required': self.required,
            'no_value_preview_text': str(self.no_value_preview_text),
            'buttonlabel': str(self.buttonlabel),
            'buttonlabel_novalue': str(self.buttonlabel_novalue),
            'usebuttonlabel': str(self.usebuttonlabel),
            'include_time': False,
            'close_icon': cradmin_icon(self.close_iconkey),
            'back_icon': cradmin_icon(self.back_iconkey),
            'destinationfield_momentjs_format': self.destinationfield_momentjs_format,
            'timeselector_datepreview_momentjs_format': self.timeselector_datepreview_momentjs_format,
            'close_screenreader_text': str(self.close_screenreader_text),
            'year_screenreader_text': str(self.year_screenreader_text),
            'month_screenreader_text': str(self.month_screenreader_text),
            'day_screenreader_text': str(self.day_screenreader_text),
            'hour_screenreader_text': str(self.hour_screenreader_text),
            'minute_screenreader_text': str(self.minute_screenreader_text),
            'usebutton_arialabel_prefix': str(self.usebutton_arialabel_prefix),
            'usebutton_arialabel_momentjs_format': str(self.usebutton_arialabel_momentjs_format),
            'back_to_datepicker_screenreader_text': str(self.back_to_datepicker_screenreader_text),
            'dateselector_table_screenreader_caption': str(self.dateselector_table_screenreader_caption),
            'today_label_text': str(self.today_label_text),
            'now_button_text': str(self.now_button_text),
            'today_button_text': str(self.today_button_text),
            'clear_button_text': str(self.clear_button_text),
            'selected_day_label_text': str(self.selected_day_label_text),
            'date_label_text': str(self.date_label_text),
            'time_label_text': str(self.time_label_text),

            'yearselect_values': list(self.get_yearselect_values()),
            'hourselect_values': list(self.get_hourselect_values()),
            'minuteselect_values': list(self.get_minuteselect_values()),

            'yearselect_momentjs_format': self.get_yearselect_momentjs_format(),
            'monthselect_momentjs_format': self.get_monthselect_momentjs_format(),
            'dayofmonthselect_momentjs_format': self.get_dayofmonthselect_momentjs_format(),
            'dayofmonthtablecell_momentjs_format': self.get_dayofmonthtablecell_momentjs_format(),
            'hourselect_momentjs_format': self.get_hourselect_momentjs_format(),
            'minuteselect_momentjs_format': self.get_minuteselect_momentjs_format(),

            'preview_change_animation_cssclass': self.get_preview_change_animation_cssclass(),
            'preview_change_animation_duration_milliseconds': self.get_preview_change_animation_duration_milliseconds(),
            'hide_animation_duration_milliseconds': self.get_hide_animation_duration_milliseconds(),

            'default_now_time': self.default_now_time

            # 'year_emptyvalue': str(self.year_emptyvalue),
            # 'month_emptyvalue': str(self.month_emptyvalue),
            # 'day_emptyvalue': str(self.day_emptyvalue),
            # 'hour_emptyvalue': str(self.hour_emptyvalue),
            # 'minute_emptyvalue': str(self.minute_emptyvalue),
        }

        if self.minimum_datetime:
            minimum_datetime = self.minimum_datetime.strftime('%Y-%m-%d %H:%M')
        else:
            minimum_datetime = None
        if self.maximum_datetime:
            maximum_datetime = self.maximum_datetime.strftime('%Y-%m-%d %H:%M')
        else:
            maximum_datetime = None
        configdict['minimum_datetime'] = minimum_datetime
        configdict['maximum_datetime'] = maximum_datetime

        return configdict

    def get_preview_angularjs_template(self):
        """
        Get the AngularJS template to use to render the preview
        of the selected date.

        Defaults to the ``preview_angularjs_template`` kwarg to ``__init__``,
        which defaults to :obj:`~.DatePickerWidget.default_preview_angularjs_template`.

        Must return a string, The string is evaluated by
        the AngularJS ``$compile`` function with the following scope:

        - ``momentObject``: A momentjs object with the value of the
           selected date/time. You use ``momentObject.format()`` to
           format the date. See the examples below and
           `moment.js docs <http://momentjs.com/docs/#/displaying/>`_.

        Examples:

            The default:

            .. code-block:: html

                {{ momentObject.format('LL') }}

            A more complex example:

            .. code-block:: html

                <span class="start-dayname">{{ momentObject.format('dddd') }}</span>
                <span class="start-monthname">{{ momentObject.format('MMMM') }}</span>
                <span class="start-dayinmonth">{{ momentObject.format('Mo') }}</span>
                <span class="start-year">{{ momentObject.format('YYYY') }}</span>

            Inserting translation strings (remember to use ``u"..."`` if using Python2)::

                return "{label}: {{ momentObject.format('LL') }}".format(
                    label=gettext_lazy('Selected date'))
        """
        return self.preview_angularjs_template

    def __get_preview_angularjs_template(self):
        """
        We use this internally instead of using :meth:`.get_preview_angularjs_template`
        to ensure the returned HTML has a root element (which is required by the
        angularjs ``$compile``-function).
        """
        return '<span>{}</span>'.format(self.get_preview_angularjs_template())

    def get_template_name(self):
        """
        Get the name of the template used to render the widget.

        Defaults to :obj:`.template_name`.
        """
        return self.template_name

    def get_extra_css_classes(self):
        """
        Get extra css classes for the wrapper around the field and widget.

        Use this if you want to customize the look of a specific widget.
        If you want to customize all datetime widgets, a custom cradmin-legacy
        theme is probably a better solution.

        Returns:
            A string with one or more css classes.
        """
        return ''

    def get_context_data(self, fieldid, rendered_field, fieldname, value, attrs):
        """
        Get context data for the template used by :meth:`.render` to render the widget.

        The template can be overridden in :meth:`.get_template_name`.
        """
        triggerbuttonid = '{}_triggerbutton'.format(fieldid)
        previewid = '{}_preview'.format(fieldid)
        previewtemplateid = '{}_previewtemplate'.format(fieldid)
        return {
            'classname_lower': self.__class__.__name__.lower(),
            'field': rendered_field,
            'fieldid': fieldid,
            'datepicker_config': json.dumps(self.get_datepicker_config(
                fieldid=fieldid,
                triggerbuttonid=triggerbuttonid,
                previewid=previewid,
                previewtemplateid=previewtemplateid,
            )),
            'triggerbuttonid': triggerbuttonid,
            'previewid': previewid,
            'previewtemplateid': previewtemplateid,
            'preview_angularjs_template': self.__get_preview_angularjs_template(),
            'extra_css_classes': self.get_extra_css_classes(),
            'widget': {
                'type': 'text',
                'name': fieldname,
                'value': value,
                'attrs': attrs
            }
        }

    def get_field_attributes(self, attributes):
        """
        Get the attributes for the field as a dict.

        We default to setting the style attribute to ``display: none;``.
        """
        attributes['style'] = 'display: none;'
        return attributes

    def render(self, name, value, attrs=None, renderer=None):
        """
        Render the widget.

        Uses the template returned by :meth:`.get_template_name` with the
        context data returned by :meth:`.get_context_data`.
        """
        attrs = attrs or {}
        attrs = self.get_field_attributes(attrs)
        if 'required' in attrs:
            attrs['required'] = False
        rendered_field = super(DatePickerWidget, self).render(name, value, attrs)
        fieldid = attrs.get('id', 'id_{}'.format(name))
        return loader.render_to_string(self.get_template_name(), self.get_context_data(
            fieldid=fieldid,
            rendered_field=rendered_field,
            fieldname=name,
            value=value,
            attrs=attrs
        ))

    #
    # Year select config
    #

    def get_yearselect_momentjs_format(self):
        """
        Get the momentjs format to use when creating the
        label for a year in the year ``<select>``.
        """
        return 'YYYY'

    def get_yearselect_values(self):
        """
        Get an iterable of year ints (I.e.: anything that can be converted to a
        list of ints with ``list(value)``).

        The iterable must yield integers.

        Defaults to the range of ~130 years in the past and future.
        """
        year_minimum_value = (timezone.now() - timedelta(days=364 * 130)).year
        year_maximum_value = (timezone.now() + timedelta(days=364 * 130)).year
        if self.minimum_datetime:
            year_minimum_value = self.minimum_datetime.year
        if self.maximum_datetime:
            year_maximum_value = self.maximum_datetime.year
        return range(
            year_minimum_value,
            year_maximum_value)

    #
    # Day select config
    #

    def get_monthselect_momentjs_format(self):
        """
        Get the momentjs format to use when creating the
        label for a month in the month ``<select>``.
        """
        return 'MMM'

    #
    # Day select config
    #

    def get_dayofmonthselect_momentjs_format(self):
        """
        Get the momentjs format to use when creating the
        label for a day in the day ``<select>``.

        Not used in the table that we use on the desktop,
        there we use :meth:`.get_dayofmonthtablecell_momentjs_format`.
        """
        return 'DD'

    #
    # Day in table config
    #

    def get_dayofmonthtablecell_momentjs_format(self):
        """
        Get the momentjs format to use when creating the
        label for a day in the table used to select a
        day on desktop.

        Not used in the ``<select>`` that we use on the desktop.
        there we use :meth:`.get_dayofmonthselect_momentjs_format`.
        """
        return 'D'

    #
    # Hour select config
    #

    def get_hourselect_momentjs_format(self):
        """
        Get the momentjs format to use when creating the
        label for an hour in the hour ``<select>``.
        """
        return 'HH'

    def get_hourselect_values(self):
        """
        Get an iterable of hour ints (I.e.: anything that can be converted to a
        list of ints with ``list(value)``).

        The iterable must yield integers.

        Defaults to a ``[0, 1, 2, ..., 23]``.
        """
        return range(0, 24)

    #
    # Minute select config
    #

    def get_minuteselect_momentjs_format(self):
        """
        Get the momentjs format to use when creating the
        label for a minute in the minute ``<select>``.
        """
        return 'mm'

    def get_minuteselect_values(self):
        """
        Get an iterable of minute ints (I.e.: anything that can be converted to a
        list of numbers with ``list(value)``).

        The iterable must yield integers.

        Defaults to ``[0, 5, 10, 15, ...55, 59]``.
        """
        minutevalues = list(range(0, 60, 5))
        minutevalues.append(59)
        return minutevalues

    def get_preview_change_animation_cssclass(self):
        """
        We use a css animation to animate the preview when the value changes.

        You can provide your own animation css class by overriding
        this.

        To disable the animation, override, and return ``None``.

        If you provide your own animation, you should also override
        :meth:`.get_preview_change_animation_duration_milliseconds`
        to ensure the duration of your css animation matches the
        time when the animation css class is removed from the preview
        element.
        """
        # return 'cradmin-legacy-datepicker-preview-changeanimate'
        return None

    def get_preview_change_animation_duration_milliseconds(self):
        """
        The duration of the preview change animation in milliseconds.

        See :meth:`.get_preview_change_animation_cssclass`.
        """
        return 2000

    def get_hide_animation_duration_milliseconds(self):
        """
        The duration of the hide animation in milliseconds.

        If you change your css to apply a different animation
        on hide, you should also override this.
        """
        return 150


class DateTimePickerWidget(DatePickerWidget):
    """
    A Widget for selecting a date and time.

    Extends :class:`.DatePickerWidget`.
    """
    default_buttonlabel = pgettext_lazy('Has value trigger button text for date widget', 'Change date/time')
    default_buttonlabel_novalue = pgettext_lazy('No value trigger button text for date widget', 'Select a date/time')

    default_preview_angularjs_template = "{{ momentObject.format('LLLL') }}"
    destinationfield_momentjs_format = 'YYYY-MM-DD HH:mm'
    usebutton_arialabel_momentjs_format = 'LLLL'

    def get_datepicker_config(self, *args, **kwargs):
        config = super(DateTimePickerWidget, self).get_datepicker_config(*args, **kwargs)
        config.update({
            'include_time': True,
        })
        return config


class TimePickerWidget(widgets.TimeInput):
    """
    Time picker widget that is wrapped in a div with the
    ``cradmin-legacy-timepicker`` css class.
    """
    template_name = 'cradmin_legacy/widgets/timepicker.django.html'

    def __init__(self, attrs=None, format=None, placeholder=gettext_lazy('hh:mm')):
        if attrs is None:
            attrs = {}
        if placeholder:
            attrs['placeholder'] = placeholder
        super(TimePickerWidget, self).__init__(attrs=attrs, format=format)

    def render(self, name, value, attrs=None, renderer=None):
        inputfield = super(TimePickerWidget, self).render(name, value, attrs)
        return loader.render_to_string(self.template_name, {
            'inputfield': inputfield,
            'fieldname': name
        })


# class DateTimePickerWidget(widgets.MultiWidget):
#     """
#     A Widget using :class:`.DatePickerWidget`
#     for selecting a date and :class:`.TimePickerWidget`
#     for selecting time.
#     """
#
#     def __init__(self, attrs=None,
#                  datewidget_placeholder=gettext_lazy('yyyy-mm-dd'),
#                  timewidget_placeholder=gettext_lazy('hh:mm')):
#         """
#         Parameters:
#             datewidget_placeholder: The placeholder for the datewidget.
#             timewidget_placeholder: The placeholder for the time widget.
#         """
#         _widgets = [
#             DatePickerWidget(attrs=attrs),
#             TimePickerWidget(attrs=attrs, placeholder=timewidget_placeholder)
#         ]
#         super(DateTimePickerWidget, self).__init__(_widgets, attrs)
#
#     def decompress(self, value):
#         return [value, value]
#
#     def format_output(self, rendered_widgets):
#         return u'<div class="cradmin-legacy-datetimepicker">{}</div>'.format(
#             u''.join(rendered_widgets))
#
#     def value_from_datadict(self, data, files, name):
#         timevalue = data.get('{}_1'.format(name), '').strip()
#         datevalue = data.get('{}_0'.format(name), '').strip()
#         values = []
#         if datevalue:
#             values.append(datevalue)
#         if timevalue:
#             values.append(timevalue)
#         datetimevalue = ' '.join(values)
#         if datetimevalue:
#             return datetimevalue
#         else:
#             return None


class SplitTimePickerWidget(widgets.MultiWidget):
    """
    Time picker widget that shows a select button is wrapped in a div with the
    ``cradmin-legacy-split-timepicker``.
    """

    def __init__(self, attrs=None,
                 hour_choices=None, minute_choices=None,
                 empty_hour_label=None, empty_minute_label=None):
        """
        Parameters:
            datewidget_placeholder: The placeholder for the datewidget.
            empty_hour_label: Set this to something other than ``None`` if you
                want to allow empty values. Adds this text for the ``None`` value
                as the first option in the select.
            empty_minute_label: Set this to something other than ``None`` if you
                want to allow empty values. Adds this text for the ``None`` value
                as the first option in the select.
        """
        hour_choices = hour_choices or self.get_hour_choices(empty_hour_label)
        minute_choices = minute_choices or self.get_minute_choices(empty_minute_label)
        _widgets = [
            WrappedSelect(attrs=attrs, choices=hour_choices,
                          wrapper_css_class='cradmin-legacy-split-timepicker-hour'),
            WrappedSelect(attrs=attrs, choices=minute_choices,
                          wrapper_css_class='cradmin-legacy-split-timepicker-minute'),
        ]
        super(SplitTimePickerWidget, self).__init__(_widgets, attrs)

    def format_hour_label(self, number):
        return '{:02}'.format(number)

    def format_minute_label(self, number):
        return '{:02}'.format(number)

    def format_hour_choice(self, number):
        return number, self.format_hour_label(number)

    def format_minute_choice(self, number):
        return number, self.format_minute_label(number)

    def get_hour_values(self):
        return range(0, 24)

    def get_minute_values(self):
        minutevalues = list(range(0, 60, 5))
        minutevalues.append(59)
        return minutevalues

    def get_hour_choices(self, emptylabel):
        choices = list(map(self.format_hour_choice, self.get_hour_values()))
        if emptylabel is not None:
            choices.insert(0, (None, emptylabel))
        return choices

    def get_minute_choices(self, emptylabel):
        choices = list(map(self.format_minute_choice, self.get_minute_values()))
        if emptylabel is not None:
            choices.insert(0, (None, emptylabel))
        return choices

    def decompress(self, value):
        if value and isinstance(value, datetime.datetime):
            return [value.hour, value.minute]
        else:
            return [None, None]

    def format_output(self, rendered_widgets):
        return u'<div class="cradmin-legacy-split-timepicker">{}</div>'.format(
            u''.join(rendered_widgets))

    def value_from_datadict(self, data, files, name):
        hourvalue = data.get('{}_0'.format(name), '').strip()
        minutevalue = data.get('{}_1'.format(name), '').strip()
        values = []
        if hourvalue:
            values.append(hourvalue)
        if minutevalue:
            values.append(minutevalue)
        datetimevalue = ':'.join(values)
        if datetimevalue:
            return datetimevalue
        else:
            return None



from django.conf import settings


def get_setting(settingvariable, fallback=None):
    return getattr(settings, settingvariable, fallback)


def get_cradmin_legacy_menu_scroll_top_fixed_setting():
    scroll_top_fixed_settings = get_setting('CRADMIN_LEGACY_MENU_SCROLL_TOP_FIXED', False)
    if scroll_top_fixed_settings is True:
        return {
            'cssClasses': {
                'scrollingClass': 'cradmin-legacy-menu-scrolling',
            }
        }
    else:
        return False


def cradmin(request):
    return {
        'CRADMIN_LEGACY_THEME_PATH': get_setting(
            'CRADMIN_LEGACY_THEME_PATH',
            'cradmin_legacy/dist/css/cradmin_theme_default/theme.css'),
        'CRADMIN_LEGACY_MENU_SCROLL_TOP_FIXED': get_cradmin_legacy_menu_scroll_top_fixed_setting(),
        'CRADMIN_LEGACY_MOMENTJS_LOCALE': get_setting(
            'CRADMIN_LEGACY_MOMENTJS_LOCALE', None),
        'CRADMIN_LEGACY_CSS_ICON_LIBRARY_PATH': get_setting(
            'CRADMIN_LEGACY_CSS_ICON_LIBRARY_PATH',
            'cradmin_legacy/dist/vendor/fonts/fontawesome/css/font-awesome.min.css'),
        'CRADMIN_LEGACY_HIDE_PAGE_HEADER': get_setting(
            'CRADMIN_LEGACY_HIDE_PAGE_HEADER', False),
    }



import json

from django import forms
from django.views.generic.edit import FormMixin

from cradmin_legacy.viewhelpers.multiselect2 import listbuilder_itemvalues
from cradmin_legacy.viewhelpers.multiselect2 import target_renderer


class ViewMixin(FormMixin):
    """
    Base class for the mixin classes for views that is shown in an iframe
    by :class:`cradmin_legacy.viewhelpers.multiselect2.manytomanywidget.Widget`.
    """
    hide_menu = True

    #: The default target renderer task. Should be a subclass of
    #: :class:`cradmin_legacy.viewhelpers.multiselect2.target_renderer.ManyToManySelectTarget`.
    target_renderer_class = target_renderer.ManyToManySelectTarget

    form_class = forms.Form

    def get_selected_values_list(self):
        """
        Returns:
            list: json decodes ``request.GET['manytomany_select_current_value']``
            and returns the result.
        """
        return json.loads(self.request.GET['manytomany_select_current_value'])

    def get_selected_objects(self):
        """
        Make a queryset from :meth:`.get_selected_values_list`.

        Supports :class:`.cradmin_legacy.viewhelpers.listfilter.listfilter_viewmixin.ViewMixin`
        out of the box, so if a ``get_unfiltered_queryset_for_role()``-method is available
        on the class, we use that instead of ``get_queryset_for_role()``.

        Returns:
            django.db.QuerySet: A queryset with the model objects matching
            :meth:`.get_selected_values_list`. Defaults to filtering
            the values with ``pk__in=<get_selected_values_list()>``.
        """
        if hasattr(self, 'get_unfiltered_queryset_for_role'):
            queryset = self.get_unfiltered_queryset_for_role(self.request.cradmin_role)
        else:
            queryset = self.get_queryset_for_role(self.request.cradmin_role)
        return queryset.filter(pk__in=self.get_selected_values_list())

    def should_include_previously_selected(self):
        """
        This should return ``True`` if we want to include the values in
        :meth:`.get_selected_objects` in the list.

        This should be ``True`` for the initial page load, which means
        that it should be ``True`` if we are one the first page, and no
        filtering/searching is applied.

        Returns:
            bool: Should we include previously selected values in the listbuilder list.
        """
        if hasattr(self, 'get_filters_string') and self.get_filters_string():
            return False
        elif getattr(self, 'paginate_by', None):
            page_kwarg = self.page_kwarg
            page = self.kwargs.get(page_kwarg) or self.request.GET.get(page_kwarg) or 1
            return str(page) == '1'
        else:
            return True

    def get_queryset_for_role(self, role=None):
        queryset = super(ViewMixin, self).get_queryset_for_role(role=role)
        if self.should_include_previously_selected():
            # We have to exclude the selected values because we add them
            # to the listbuilder list as selected items in :meth:`.get_listbuilder_list`.
            queryset = queryset.exclude(pk__in=self.get_selected_values_list())
        return queryset

    def get_target_renderer_class(self):
        """
        Returns:
            cradmin_legacy.viewhelpers.multiselect2.target_renderer.ManyToManySelectTarget: The target renderer
            class.
        """
        return self.target_renderer_class

    def get_target_renderer_kwargs(self):
        """
        Returns:
            dict: Keyword arguments for the constructor of :meth:`.get_target_renderer_class`.
        """
        return {
            'target_formfield_id': self.request.GET['manytomany_select_fieldid'],
            'empty_selection_allowed': self.request.GET['manytomany_select_required'] != 'True',
            'form': self.get_form(form_class=self.get_form_class())
        }

    def get_target_renderer(self):
        """
        Get a select target renderer. Defaults to returning the
        :meth:`.get_target_renderer_class` initialized with
        :meth:`.get_target_renderer_kwargs`.
        """
        targer_renderer_class = self.get_target_renderer_class()
        return targer_renderer_class(**self.get_target_renderer_kwargs())


class ListBuilderViewMixin(ViewMixin):
    """
    Mixin class for using a listbuilder view as a select view for
    :class:`cradmin_legacy.viewhelpers.multiselect2.manytomanywidget.Widget`.

    You only need to render the target renderable returned by
    :meth:`.ViewMixin.get_target_renderer`. You will typically
    do this by adding it in ``get_context_data()``, and render it with
    :meth:`cradmin_legacy.templatetags.cradmin_legacy_tags.cradmin_render_renderable`.

    Examples:

        Simple example::

            from cradmin_legacy.viewhelpers import listbuilderview
            from cradmin_legacy.viewhelpers import multiselect2

            class MySelectView(multiselect2.manytomanyview.ListBuilderViewMixin,
                               listbuilderview.FilterListMixin,
                               listbuilderview.View):
                def get_queryset_for_role(self, role):
                    ...

                def get_context_data(self, context):
                    context = super(MySelectView, self).get_context_data(context)
                    context['target_renderer'] = self.get_target_renderer()
                    return context
    """
    value_renderer_class = listbuilder_itemvalues.ManyToManySelect

    def get_listbuilder_list(self, context):
        listbuilder_list = super(ListBuilderViewMixin, self).get_listbuilder_list(context=context)
        if self.should_include_previously_selected():
            value_renderer_class = self.get_value_renderer_class()
            frame_renderer_class = self.get_frame_renderer_class()
            listbuilder_list.extend_with_values(
                value_iterable=self.get_selected_objects(),
                value_renderer_class=value_renderer_class,
                frame_renderer_class=frame_renderer_class,
                value_and_frame_renderer_kwargs={
                    'is_selected': True
                })
        return listbuilder_list


class ListBuilderFilterListViewMixin(ListBuilderViewMixin):
    """
    Mixin class for using a listbuilder view with filterlist as a select view for
    :class:`cradmin_legacy.viewhelpers.multiselect2.manytomanywidget.Widget`.

    You only need to render the target renderable returned by
    :meth:`.ViewMixin.get_target_renderer`. You will typically
    do this by adding it in ``get_context_data()``, and render it with
    :meth:`cradmin_legacy.templatetags.cradmin_legacy_tags.cradmin_render_renderable`.


    Examples:

        Simple example::

            from cradmin_legacy.viewhelpers import listbuilderview
            from cradmin_legacy.viewhelpers import multiselect2

            class MySelectView(multiselect2.manytomanyview.ListBuilderFilterListViewMixin,
                               listbuilderview.FilterListMixin,
                               listbuilderview.View):
                def get_queryset_for_role(self, role):
                    ...
    """
    def get_filterlist_url(self, filters_string):
        """
        Implements :meth:`cradmin_legacy.viewhelpers.listfilter.listfilter_viewmixin.ViewMixin#get_filterlist_url`
        with a default that assumes you have a view named ``manytomanyselect-filter``
        that takes ``filters_string`` as kwarg.
        """
        return self.request.cradmin_app.reverse_appurl(
            'manytomanyselect-filter', kwargs={'filters_string': filters_string})

    def add_target_renderer_to_filterlist(self, filterlist):
        """
        Add the :meth:`~.ListBuilderViewMixin.get_target_renderer` to the filter list.
        """
        filterlist.append(self.get_target_renderer())

    def add_filterlist_items(self, filterlist):
        """
        Automatically adds the :meth:`~.ListBuilderViewMixin.get_target_renderer`
        to the filter list.

        Uses :meth:`.add_target_renderer_to_filterlist` to add the target renderer,
        so if you want to add the target renderer some other place, you
        can just override that method in a subclass.
        """
        super(ListBuilderFilterListViewMixin, self).add_filterlist_items(filterlist=filterlist)
        self.add_target_renderer_to_filterlist(filterlist=filterlist)



from __future__ import unicode_literals

from django.contrib.contenttypes.fields import GenericForeignKey
from future import standard_library

from cradmin_legacy import automodelform
from cradmin_legacy.crispylayouts import PrimarySubmit, DefaultSubmit

standard_library.install_aliases()
import urllib.request
import urllib.parse
import urllib.error
from django import forms
from django import http
from django.contrib import messages
from django.core import serializers
from django.utils.translation import gettext_lazy

from crispy_forms import layout

from . import formbase


class CreateUpdateViewMixin(formbase.FormViewMixin):
    """
    Mixin class for Update and Create views.
    """

    #: The model class.
    model = None

    #: List of field names.
    #: See :meth:`.get_model_fields` and :meth:`.get_field_layout`.
    fields = None

    #: The name of the submit button used for preview.
    #: Only used when :meth:`.preview_url` is defined.
    submit_preview_name = 'submit-preview'

    #: The field that should always be set to the current role.
    #: Removes the field from the form (see :meth:`.get_form`),
    #: and instead sets the value directly on the object in
    #: :meth:`.save_object`.
    roleid_field = None

    #: The viewname within this app for the edit view.
    #: See :meth:`.get_editurl`.
    editview_appurl_name = 'edit'

    def get_model_class(self):
        """
        Get the model class.

        Defaults to :obj:`~.CreateUpdateViewMixin.model`.
        """
        return self.model

    def make_default_form_class(self):
        """
        Make a ModelForm class with the model set to
        :meth:`.get_model_class` and the fields set to
        :meth:`.get_model_fields`.
        """
        me = self

        class Form(automodelform.ModelForm):
            class Meta:
                model = me.get_model_class()
                fields = me.get_model_fields()

        return Form

    def get_form_class(self):
        if self.form_class:
            return self.form_class
        else:
            return self.make_default_form_class()

    def get_form_kwargs(self):
        kwargs = super(CreateUpdateViewMixin, self).get_form_kwargs()
        form_class = self.get_form_class()
        if issubclass(form_class, automodelform.ModelForm):
            kwargs['view'] = self
        return kwargs

    def add_preview_button_if_configured(self, buttons):
        preview_url = self.get_preview_url()
        if preview_url:
            buttons.append(DefaultSubmit(self.submit_preview_name, gettext_lazy('Preview')))

    def get_model_fields(self):
        """
        Get model fields. Defaults to :obj:`.fields`.
        """
        if self.fields:
            return self.fields
        else:
            fields = list(forms.fields_for_model(self.get_model_class(), fields=self.fields).keys())
            if self.roleid_field and self.roleid_field in fields:
                fields.remove(self.roleid_field)
            return fields

    def get_field_layout(self):
        """
        Get a list/tuple of fields. These are added to a ``crispy_forms.layout.Layout``.
        Defaults to a all fields on the model. If :obj:`.fields`, we use those fields.

        Simple example (same as specifying the fields in :obj:`.fields`)::

            from crispy_forms import layout

            class MyCreateView(CreateView):
                def get_field_layout(self):
                    return [
                        layout.Div(
                            'title', 'name', 'size', 'tags',
                            css_class='cradmin-globalfields')
                    ]

        A slightly more complex example::

            from crispy_forms import layout

            class MyCreateView(CreateView):
                def get_field_layout(self):
                    return [
                        layout.Div('title', css_class="cradmin-focusfield cradmin-focusfield-lg"),
                        layout.Div('description', css_class="cradmin-focusfield"),
                        layout.Fieldset('Metadata',
                            'size',
                            'tags'
                        )

                    ]

        """
        fields = self.get_model_fields()
        return [layout.Div(*fields, css_class='cradmin-globalfields')]

    def get_form(self, form_class=None):
        """
        If you set :obj:`.roleid_field`, we will remove that field from
        the form.

        .. note::
            The :obj:`.roleid_field` handling also works for GenericForeignKey
            fields (removes the content type and object pk field from the form).
        """
        form = super(CreateUpdateViewMixin, self).get_form(form_class)
        if self.roleid_field:
            roleid_fieldobject = getattr(self.get_model_class(), self.roleid_field, None)
            if isinstance(roleid_fieldobject, GenericForeignKey):
                for fieldname in roleid_fieldobject.fk_field, roleid_fieldobject.ct_field:
                    if fieldname in form.fields:
                        del form.fields[fieldname]
            else:
                if self.roleid_field in form.fields:
                    del form.fields[self.roleid_field]
        return form

    def get_context_data(self, **kwargs):
        context = super(CreateUpdateViewMixin, self).get_context_data(**kwargs)
        context['model_verbose_name'] = self.get_model_class()._meta.verbose_name
        return context

    def get_editurl(self, obj):
        """
        Get the edit URL for ``obj``.

        Defaults to::

            self.request.cradmin_app.reverse_appurl(self.editview_appurl_name, args=[obj.pk])
        """
        return self.request.cradmin_app.reverse_appurl(self.editview_appurl_name, args=[obj.pk])

    def _get_full_editurl(self, obj):
        url = self.get_editurl(obj)
        if 'success_url' in self.request.GET:
            url = '{}?{}'.format(
                url, urllib.parse.urlencode({
                    'success_url': self.request.GET['success_url']}))
        return url

    def get_success_url(self):
        if self.get_submit_save_and_continue_edititing_button_name() in self.request.POST:
            return str(self._get_full_editurl(self.object))
        else:
            return str(self.get_default_save_success_url())

    def set_automatic_attributes(self, obj):
        """
        Called by :meth:`.save_object` to set automatic attributes for the
        object before it is saved.

        This is where we handle :obj:`.roleid_field`, but you can override this
        to set your own automatic attributes. Just remember to call ``super``
        if you want to keep the :obj:`.roleid_field` magic.

        Parameters:
            obj: The object you are about to save.
        """
        if self.roleid_field:
            setattr(obj, self.roleid_field, self.request.cradmin_role)

    def save_object(self, form, commit=True):
        """
        Save the object. You can override this to customize how the
        form is turned into a saved object.

        Make sure you call ``super`` if you override this (see the docs for the commit parameter).
        If you do not, you will loose the automatic handling of obj:`.roleid_field`.

        Parameters:
            commit (boolean): If this is ``False``, the object is returned
                unsaved. Very useful when you want to manipulate the object
                before saving it in a subclass.

        Returns:
            The saved object.
        """
        obj = form.save(commit=False)
        self.set_automatic_attributes(obj=obj)
        if commit:
            obj = form.save(commit=True)
        return obj

    def form_valid(self, form):
        """
        If the form is valid, save the associated model.
        """
        if self.preview_requested():
            self.store_preview_in_session(self.serialize_preview(form))
            return self.render_to_response(self.get_context_data(form=form, show_preview=True))
        else:
            self.object = self.save_object(form)
            self.form_saved(self.object)
            self.add_success_messages(self.object)
            return http.HttpResponseRedirect(self.get_success_url())

    def form_saved(self, object):
        """
        Called after the form has been successfully saved.
        The ``object`` is the saved object.

        Does nothing by default, but you can override it if you need to
        do something extra post save.
        """
        pass

    def get_success_message(self, object):
        """
        Override this to provide a success message.

        The ``object`` is the saved object.

        Used by :meth:`.add_success_messages`.
        """
        return None

    def add_success_messages(self, object):
        """
        Called after the form has been saved, and after :meth:`.form_saved` has been called.

        The ``object`` is the saved object.

        Defaults to add :meth:`.get_success_message` as a django messages
        success message if :meth:`.get_success_message` returns anything.

        You can override this to add multiple messages or to show messages in some other way.
        """
        success_message = self.get_success_message(object)
        if success_message:
            messages.success(self.request, success_message)

    def get_form_invalid_message(self, form):
        """
        You can override this to provide a custom error message.

        Defaults to "Please fix the errors in the form below.".

        The ``form`` is the invalid form object.

        Used by :meth:`.add_form_invalid_messages`.
        """
        return gettext_lazy('Please fix the errors in the form below.')

    def add_form_invalid_messages(self, form):
        """
        Called to add messages when the form does not validate.

        The ``form`` is the invalid form object.

        Defaults to add :meth:`.get_form_invalid_message` as a django messages
        error message if :meth:`.get_form_invalid_message` returns anything.

        You can override this to add multiple messages or to show error messages in some other way.
        """
        form_invalid_message = self.get_form_invalid_message(object)
        if form_invalid_message:
            messages.error(self.request, form_invalid_message)

    def form_invalid(self, form):
        self.add_form_invalid_messages(form)
        return super(CreateUpdateViewMixin, self).form_invalid(form)

    def serialize_preview(self, form):
        """
        Seralize for preview.

        Defaults to serializing the object as JSON using ``django.core.serializers``.
        You can safely override this, but you will also have to override
        :meth:`deserialize_preview`.
        """
        return serializers.serialize('json', [self.save_object(form, commit=False)])

    @classmethod
    def deserialize_preview(cls, serialized):
        """
        Deseralize a preview serialized with :meth:`.serialize_preview`.

        You must override this and :meth:`.serialize_preview` - they work together
        to send the preview to the preview View.
        """
        return list(serializers.deserialize('json', serialized))[0].object

    @classmethod
    def get_preview_sessionkey(cls):
        """
        Get the session key used for preview. You should not
        need to override this.

        Unlike the default implementation of this method from
        :class:`cradmin_legacy.viewhelpers.formbase.PreviewMixin`,
        we use the model class module and name as the session key.
        This is simply because we do not want it to matter if
        you fetch preview data from create or update views
        for the same model (to simplify implementing preview views).
        """
        sessionkey = 'cradmin_legacy__{module}.{classname}'.format(
            module=cls.model.__module__,
            classname=cls.model.__name__)
        return sessionkey


class OnlySaveButtonMixin(object):
    """
    Mixin class that overrides ``get_buttons()`` to just include
    the save button (no "Save and continue editing").

    You must mix this in **before** any :class:`.CreateUpdateViewMixin`
    subclass (like UpdateView or CreateView).
    """
    def get_buttons(self):
        if hasattr(self, 'is_in_foreignkey_select_mode') and self.is_in_foreignkey_select_mode():
            return super(OnlySaveButtonMixin, self).get_buttons()
        buttons = [
            PrimarySubmit(self.get_submit_save_button_name(), self.get_submit_save_label()),
        ]
        self.add_preview_button_if_configured(buttons)
        return buttons



import re
from collections import OrderedDict

from django.apps import apps
from django.utils.translation import gettext_lazy
from future.utils import python_2_unicode_compatible

from cradmin_legacy import crinstance
from cradmin_legacy import crmenu
from cradmin_legacy import crapp
from cradmin_legacy.crinstance import reverse_cradmin_url
from cradmin_legacy.crsettings import get_setting
from cradmin_legacy.decorators import has_access_to_cradmin_instance
from cradmin_legacy.superuserui.crapps import djangomodel
from cradmin_legacy.superuserui.crapps import djangoapp
from cradmin_legacy.superuserui.views import dashboardview


@python_2_unicode_compatible
class ModelConfig(object):
    def __init__(self, model_class=None, menulabel=None, crapp_class=None):
        self.model_class = model_class
        self.menulabel = menulabel
        self.crapp_class = crapp_class
        self.djangoappconfig = None  # This is set in DjangoAppConfig.add_model()

    def get_model_class(self):
        if self.model_class:
            return self.model_class
        else:
            raise NotImplementedError('You must override get_model_class() or provide the '
                                      '``model_class`` parameter for __init__().')

    def get_menulabel(self):
        if self.menulabel:
            return self.menulabel
        else:
            return self.get_model_class()._meta.verbose_name_plural.capitalize()

    def get_view_url(self, viewname, args=None, kwargs=None):
        return reverse_cradmin_url(
            instanceid=self.djangoappconfig.registry.id,
            appname=self.get_unique_identifier(),
            viewname=viewname,
            args=args,
            kwargs=kwargs)

    def get_indexview_url(self):
        return self.get_view_url(viewname=crapp.INDEXVIEW_NAME)

    def get_foreignkeyselectview_url(self):
        return self.get_view_url(viewname='foreignkeyselect')

    def get_manytomanyselectview_url(self):
        return self.get_view_url(viewname='manytomanyselect')

    def get_unique_identifier(self):
        return '{}_{}'.format(self.model_class._meta.app_label,
                              self.model_class._meta.model_name)

    def get_crapp_class(self):
        if self.crapp_class:
            return self.crapp_class
        else:
            return djangomodel.DjangoModelCrApp

    def make_crapp_class(self):
        me = self

        class App(self.get_crapp_class()):
            modelconfig = me

        return App

    def __str__(self):
        return self.get_menulabel()


class DjangoAppConfig(object):
    def __init__(self, app_label=None, menulabel=None, crapp_class=None):
        self.app_label = app_label
        self.menulabel = menulabel
        self.crapp_class = crapp_class
        self._modelconfigs = OrderedDict()
        self.registry = None  # This is set in Registry.add_djangoapp()

    def get_app_label(self):
        if self.app_label:
            return self.app_label
        else:
            raise NotImplementedError('You must override get_app_label() or provide the '
                                      '``app_label`` parameter for __init__().')

    def get_indexview_url(self):
        return reverse_cradmin_url(
            instanceid=self.registry.id,
            appname=self.get_app_label())

    def get_menulabel(self):
        if self.menulabel:
            return self.menulabel
        else:
            return self.get_app_config().verbose_name.capitalize()

    def add_model(self, modelconfig):
        self._modelconfigs[modelconfig.get_model_class()] = modelconfig
        modelconfig.djangoappconfig = self
        self.registry._register_modelconfig(modelconfig=modelconfig)

    def get_app_config(self):
        return apps.get_app_config(self.get_app_label())

    def add_all_models(self):
        def get_model_verbose_name_plural(model_class):
            return model_class._meta.verbose_name_plural

        for model_class in sorted(self.get_app_config().get_models(),
                                  key=get_model_verbose_name_plural):
            self.add_model(ModelConfig(model_class=model_class))

    def iter_modelconfigs(self):
        return iter(self._modelconfigs.values())

    def get_crapp_class(self):
        if self.crapp_class:
            return self.crapp_class
        else:
            return djangoapp.DjangoAppCrApp

    def make_crapp_class(self):
        me = self

        class App(self.get_crapp_class()):
            djangoappconfig = me

        return App


class Registry(object):
    def __init__(self, id, urlpath_regex=r'^/superuser.*$'):
        self.id = id
        self.urlpath_regex = urlpath_regex
        self._djangoappconfigs = OrderedDict()
        self._modelconfigmap = {}

    def get_title(self):
        return gettext_lazy('Superuser UI')

    def iter_djangoappconfigs(self):
        return iter(self._djangoappconfigs.values())

    def add_djangoapp(self, djangoappconfig):
        self._djangoappconfigs[djangoappconfig.get_app_label()] = djangoappconfig
        djangoappconfig.registry = self
        return djangoappconfig

    def _register_modelconfig(self, modelconfig):
        """
        Called by :meth:`.DjangoAppConfig.add_model`.
        """
        self._modelconfigmap[modelconfig.get_model_class()] = modelconfig

    def make_menu_class(self):
        me = self

        class Menu(crmenu.Menu):
            def add_modelconfigs(self, appconfig, appmenuitem):
                for modelconfig in appconfig.iter_modelconfigs():
                    appmenuitem.add_childitem(
                        label=modelconfig.get_menulabel(),
                        url=self.appindex_url(modelconfig.get_unique_identifier()),
                        active=self.request.cradmin_app.appname == modelconfig.get_unique_identifier())

            def add_appconfig(self, appconfig):
                childappnames = set()
                for modelconfig in appconfig.iter_modelconfigs():
                    childappnames.add(modelconfig.get_unique_identifier())
                appmenuitem = self.add_menuitem(
                    label=appconfig.get_menulabel(),
                    url=appconfig.get_indexview_url(),
                    active=self.request.cradmin_app.appname == appconfig.get_app_label(),
                    expanded=self.request.cradmin_app.appname in childappnames)
                self.add_modelconfigs(appconfig=appconfig, appmenuitem=appmenuitem)

            def build_menu(self):
                for appconfig in me.iter_djangoappconfigs():
                    self.add_appconfig(appconfig=appconfig)

        return Menu

    def get_dashboardview_class(self):
        return dashboardview.View

    def get_modelconfig_for_model_class(self, model_class):
        return self._modelconfigmap.get(model_class, None)

    def make_cradmin_instance_class(self):  # noqa
        me = self

        class CrAdminInstance(crinstance.BaseCrAdminInstance):
            id = me.id
            menuclass = me.make_menu_class()

            def has_access(self):
                return self.request.user.is_superuser

            def get_cradmin_theme_path(self):
                return get_setting('CRADMIN_LEGACY_SUPERUSERUI_THEME_PATH', None)

            def get_superuserui_registry(self):
                return me

            def get_foreignkeyselectview_url(self, model_class):
                modelconfig = me.get_modelconfig_for_model_class(model_class=model_class)
                if modelconfig:
                    return modelconfig.get_foreignkeyselectview_url()
                else:
                    return None

            def get_manytomanyselectview_url(self, model_class):
                modelconfig = me.get_modelconfig_for_model_class(model_class=model_class)
                if modelconfig:
                    return modelconfig.get_manytomanyselectview_url()
                else:
                    return None

            @classmethod
            def get_apps(cls):
                apps = []
                for appconfig in me.iter_djangoappconfigs():
                    apps.append((appconfig.get_app_label(),
                                 appconfig.make_crapp_class()))
                    for modelconfig in appconfig.iter_modelconfigs():
                        apps.append((modelconfig.get_unique_identifier(),
                                     modelconfig.make_crapp_class()))
                return apps

            @classmethod
            def matches_urlpath(cls, urlpath):
                return re.match(me.urlpath_regex, urlpath)

            @classmethod
            def get_instance_frontpage_view(cls):
                return has_access_to_cradmin_instance(me.get_dashboardview_class().as_view())

        return CrAdminInstance


#: The default superuserui registry.
default = Registry(id='cradmin_legacy_superuserui_default')



from . import listbuilder_itemvalues  # noqa
from . import target_renderer  # noqa
from . import manytomanyview  # noqa
from . import manytomanywidget  # noqa
from . import selected_item_renderer  # noqa



from __future__ import unicode_literals
import os
from django.core.exceptions import ValidationError
from django.core.files.base import ContentFile
from django.test import TestCase
from model_bakery import baker
from cradmin_legacy.apps.cradmin_temporaryfileuploadstore.models import TemporaryFileCollection, TemporaryFile, \
    html_input_accept_match, truncate_filename, make_unique_filename
from cradmin_legacy.tests.helpers import create_user


class TestModels(TestCase):

    def test_delete_removes_physical_file(self):
        collection = TemporaryFileCollection.objects.create(
            user=create_user('testuser'))
        temporaryfile = TemporaryFile(filename='test.txt', collection=collection)
        temporaryfile.file.save('test.txt', ContentFile('Testdata'))
        physical_file_path = temporaryfile.file.path
        self.assertTrue(os.path.exists(physical_file_path))
        temporaryfile.file.delete()
        temporaryfile.delete()
        self.assertFalse(os.path.exists(physical_file_path))

    def test_clear_collection(self):
        collection = TemporaryFileCollection.objects.create(
            user=create_user('testuser'))
        temporaryfile = TemporaryFile(filename='test.txt', collection=collection)
        temporaryfile.file.save('test.txt', ContentFile('Testdata'))
        physical_file_path = temporaryfile.file.path
        self.assertTrue(os.path.exists(physical_file_path))
        collection.clear_files()
        self.assertFalse(os.path.exists(physical_file_path))
        self.assertEqual(collection.files.count(), 0)

    def test_singlemode_keeps_only_last_file(self):
        collection = TemporaryFileCollection.objects.create(
            user=create_user('testuser'),
            singlemode=True)

        first_added_temporary_file = TemporaryFile(filename='test1.txt', collection=collection)
        first_added_temporary_file.file.save('test1.txt', ContentFile('Testdata'), save=False)
        first_added_temporary_file.clean()
        first_added_temporary_file.save()

        last_added_temporary_file = TemporaryFile(filename='test2.txt', collection=collection)
        last_added_temporary_file.file.save('test1.txt', ContentFile('Testdata'), save=False)
        last_added_temporary_file.clean()
        last_added_temporary_file.save()
        self.assertEqual(collection.files.count(), 1)
        self.assertEqual(collection.files.first(), last_added_temporary_file)

    def test_singlemode_keeps_only_last_file_delete_physical_file(self):
        collection = TemporaryFileCollection.objects.create(
            user=create_user('testuser'),
            singlemode=True)

        first_added_temporary_file = TemporaryFile(filename='test1.txt', collection=collection)
        first_added_temporary_file.file.save('test1.txt', ContentFile('Testdata'), save=False)
        first_added_temporary_file.clean()
        first_added_temporary_file.save()
        first_added_physical_file_path = first_added_temporary_file.file.path
        self.assertTrue(os.path.exists(first_added_physical_file_path))

        last_added_temporary_file = TemporaryFile(filename='test2.txt', collection=collection)
        last_added_temporary_file.file.save('test1.txt', ContentFile('Testdata'), save=False)
        last_added_temporary_file.clean()
        self.assertFalse(os.path.exists(first_added_physical_file_path))

    def test_singlemode_keeps_only_last_file_do_not_delete_last(self):
        collection = TemporaryFileCollection.objects.create(
            user=create_user('testuser'),
            singlemode=True)

        last_added_temporary_file = TemporaryFile(filename='test2.txt', collection=collection)
        last_added_temporary_file.file.save('test1.txt', ContentFile('Testdata'), save=False)
        last_added_temporary_file.clean()
        last_added_temporary_file.save()
        last_added_temporary_file.clean()
        self.assertEqual(collection.files.count(), 1)
        self.assertEqual(collection.files.first(), last_added_temporary_file)

    def test_get_filename_set(self):
        collection = TemporaryFileCollection.objects.create(
            user=create_user('testuser'))
        temporaryfile1 = TemporaryFile(filename='test.txt', collection=collection)
        temporaryfile1.file.save('x', ContentFile('Testdata'))
        temporaryfile2 = TemporaryFile(filename='test2.txt', collection=collection)
        temporaryfile2.file.save('y', ContentFile('Testdata'))
        temporaryfile3 = TemporaryFile(filename='test.txt', collection=collection)
        temporaryfile3.file.save('z', ContentFile('Testdata'))
        self.assertEqual(collection.get_filename_set(), {'test.txt', 'test2.txt'})

    def test_set_mimetype_from_filename(self):
        temporaryfile = TemporaryFile(filename='test.png')
        temporaryfile.set_mimetype_from_filename()
        self.assertEqual(temporaryfile.mimetype, 'image/png')

    def test_set_mimetype_from_filename_no_extension(self):
        temporaryfile = TemporaryFile(filename='test')
        temporaryfile.set_mimetype_from_filename()
        self.assertEqual(temporaryfile.mimetype, '')

    def test_set_mimetype_from_filename_unknown_extension(self):
        temporaryfile = TemporaryFile(filename='test.thisdoesnotexist')
        temporaryfile.set_mimetype_from_filename()
        self.assertEqual(temporaryfile.mimetype, '')

    def clean_sets_mimetype(self):
        temporaryfile = TemporaryFile(filename='test.png')
        temporaryfile.clean()
        self.assertEqual(temporaryfile.mimetype, 'image/png')

    def clean_truncates_filename(self):
        collection = TemporaryFileCollection(max_filename_length=6)
        temporaryfile = TemporaryFile(filename='test.png', collection=collection)
        temporaryfile.clean()
        self.assertEqual(temporaryfile.filename, 'st.png')

    def clean_does_not_truncate_filename_if_max_filename_length_is_none(self):
        collection = TemporaryFileCollection(max_filename_length=None)
        temporaryfile = TemporaryFile(filename='test.png', collection=collection)
        temporaryfile.clean()
        self.assertEqual(temporaryfile.filename, 'test.png')

    def clean_validates_accept(self):
        collection = TemporaryFileCollection.objects.create(
            user=create_user('testuser'),
            accept='image/png,image/jpeg')
        TemporaryFile(filename='test.png', collection=collection).clean()
        TemporaryFile(filename='test.jpg', collection=collection).clean()
        with self.assertRaises(ValidationError):
            TemporaryFile(filename='test.txt', collection=collection).clean()

    def test_html_input_accept_match_filename_star(self):
        self.assertTrue(html_input_accept_match(accept='*.jpg', mimetype='unused', filename='test.jpg'))
        self.assertFalse(html_input_accept_match(accept='*.jpg', mimetype='unused', filename='test.jpeg'))
        self.assertTrue(html_input_accept_match(accept='.unused,*.jpg,.unused',
                                                mimetype='unused', filename='test.jpg'))

    def test_html_input_accept_match_filename_dot(self):
        self.assertTrue(html_input_accept_match(accept='.jpg', mimetype='unused', filename='test.jpg'))
        self.assertFalse(html_input_accept_match(accept='.jpg', mimetype='unused', filename='test.jpeg'))
        self.assertTrue(html_input_accept_match(accept='.unused,.jpg,.unused',
                                                mimetype='unused', filename='test.jpg'))

    def test_html_input_accept_match_mimetype_exact(self):
        self.assertTrue(html_input_accept_match(accept='image/jpeg', mimetype='image/jpeg', filename='unused'))
        self.assertFalse(html_input_accept_match(accept='image/png', mimetype='image/jpeg', filename='unused'))
        self.assertTrue(html_input_accept_match(accept='.unused,image/jpeg,unused2',
                                                mimetype='image/jpeg', filename='unused'))

    def test_html_input_accept_match_mimetype_star(self):
        self.assertTrue(html_input_accept_match(accept='image/*', mimetype='image/jpeg', filename='unused'))
        self.assertFalse(html_input_accept_match(accept='application/*', mimetype='image/jpeg', filename='unused'))
        self.assertTrue(html_input_accept_match(accept='.unused,image/*,.unused2',
                                                mimetype='image/jpeg', filename='unused'))

    def test_html_input_accept_match_mimetype_none(self):
        self.assertFalse(html_input_accept_match(accept='image/jpeg', mimetype=None, filename='test.jpeg'))

    def test_truncate_filename_not_above_max_length(self):
        self.assertEqual(truncate_filename('test.txt', maxlength=8), 'test.txt')

    def test_truncate_filename_veryshort_maxlength(self):
        self.assertEqual(truncate_filename('test.txt', maxlength=5), 't.txt')

    def test_truncate_filename_ellipsis_even_filelength_more_in_tail(self):
        self.assertEqual(truncate_filename('abcdefghABCDEFGH', maxlength=14), 'abcde...CDEFGH')

    def test_truncate_filename_ellipsis_odd_filelength(self):
        self.assertEqual(truncate_filename('abcdeX___Xhijkl', maxlength=13), 'abcde...hijkl')

    def test_make_unique_filename_already_unique(self):
        self.assertEqual(
            make_unique_filename(filename_set={'test.txt'}, wanted_filename='test2.txt'),
            'test2.txt')

    def test_make_unique_filename(self):
        self.assertNotEqual(
            make_unique_filename(filename_set={'test.txt'}, wanted_filename='test.txt'),
            'test.txt')
        self.assertTrue(
            make_unique_filename(filename_set={'test.txt'}, wanted_filename='test.txt')
            .endswith('-test.txt'))

    def test_make_unique_filename_maxlength(self):
        unique_filename = make_unique_filename(
            filename_set={'t' * 45},
            wanted_filename='t' * 45,
            max_filename_length=45)
        self.assertNotEqual(unique_filename, 't' * 45)
        self.assertTrue(unique_filename.endswith('-tttttttt'))
        self.assertEqual(len(unique_filename), 45)

    def test_no_max_filesize_bytes(self):
        collection = baker.make('cradmin_temporaryfileuploadstore.TemporaryFileCollection')
        temporaryfile = TemporaryFile(filename='test.txt', collection=collection)
        temporaryfile.file.save('test.txt', ContentFile('Testdata'))
        temporaryfile.clean()  # No ValidationError

    def test_max_filesize_bytes_size_below_ok(self):
        collection = baker.make('cradmin_temporaryfileuploadstore.TemporaryFileCollection',
                                max_filesize_bytes=100)
        temporaryfile = TemporaryFile(filename='test.txt', collection=collection)
        temporaryfile.file.save('test.txt', ContentFile('Testdata'))
        temporaryfile.clean()  # No ValidationError

    def test_max_filesize_bytes_size_above_fails(self):
        collection = baker.make('cradmin_temporaryfileuploadstore.TemporaryFileCollection',
                                max_filesize_bytes=1)
        temporaryfile = TemporaryFile(filename='test.txt', collection=collection)
        temporaryfile.file.save('test.txt', ContentFile('Testdata'))
        with self.assertRaisesMessage(ValidationError,
                                      'Can not upload files larger than 1B.'):
            temporaryfile.clean()



from __future__ import unicode_literals
from django.conf import settings
from django.template import defaultfilters
from django.views.generic import ListView
from cradmin_legacy.viewhelpers import listbuilder
from django.utils.translation import gettext_lazy
from cradmin_legacy.viewhelpers.listfilter import listfilter_viewmixin
from cradmin_legacy.viewhelpers import listfilter


class ViewMixin(object):
    """
    Listbuilder view mixin. Must be mixin in before any Django View subclass.

    This is typically used with a Django ListView or TemplateView.
    The mixin is not dependent on any specific backend, so it works
    no matter where you get your data from (database, mongodb, elasticsearch, ...).

    For a ready to use view that extends this to work with Django model objects,
    see :class:`.View`.

    The ViewMixin works much like :class:`.View`, but you must override/implement:

    - :meth:`.get_pagetitle`
    - :meth:`.get_listbuilder_list_value_iterable`
    - :meth:`.get_no_items_message`
    """
    template_name = 'cradmin_legacy/viewhelpers/listbuilderview/default.django.html'

    #: See :meth:`~ViewMixin.get_listbuilder_class`.
    listbuilder_class = listbuilder.lists.RowList

    #: See :meth:`~ViewMixin.get_value_renderer_class`.
    value_renderer_class = listbuilder.itemvalue.FocusBox

    #: See :meth:`~ViewMixin.get_frame_renderer_class`.
    frame_renderer_class = listbuilder.itemframe.DefaultSpacingItemFrame

    #: Set this to True to hide the page header. See :meth:`~.FormViewMixin.get_hide_page_header`.
    hide_page_header = False

    #: Enable previews? See :meth:`.get_enable_previews`. Defaults to ``False``.
    enable_previews = False

    def get_pagetitle(self):
        """
        Get the page title (the title tag).

        Must be implemented in subclasses.
        """
        raise NotImplementedError()

    def get_pageheading(self):
        """
        Get the page heading.

        Defaults to :meth:`.get_pagetitle`.
        """
        return self.get_pagetitle()

    def get_hide_page_header(self):
        """
        Return ``True`` if we should hide the page header.

        You can override this, or set :obj:`.hide_page_header`, or hide the page header
        in all form views with the ``CRADMIN_LEGACY_HIDE_PAGEHEADER_IN_LISTVIEWS`` setting.
        """
        return self.hide_page_header or getattr(settings, 'CRADMIN_LEGACY_HIDE_PAGEHEADER_IN_LISTVIEWS', False)

    def get_enable_previews(self):
        """
        If this returns ``True``, we enable previews.

        This is required for the ``cradmin-legacy-page-preview-open-on-click``
        angularJS directive to work.

        Defaults to :obj:`.enable_previews`.
        """
        return self.enable_previews

    def get_listbuilder_class(self):
        """
        Get a subclass of :class:`cradmin_legacy.viewhelpers.listbuilder.base.List`.

        Defaults to :obj:`.ViewMixin.listbuilder_class`.
        """
        return self.listbuilder_class

    def get_listbuilder_list_kwargs(self):
        """
        Get kwargs for :meth:`.get_listbuilder_class`.
        """
        return {}

    def get_value_and_frame_renderer_kwargs(self):
        """
        Get kwargs for the classes returned by :meth:`.get_value_renderer_class`
        and :meth:`.get_frame_renderer_class`.

        Both the frame and value renderer base classes takes ``**kwargs``
        and store them in ``self.kwargs`` (just like django class based views),
        so even if you just need to send kwargs to one of these classes,
        you can still use this.

        See the docs for the ``value_and_frame_renderer_kwargs`` argument for
        :meth:`cradmin_legacy.viewhelpers.listbuilder.base.List.extend_with_values`
        method for more information. HINT: This can also return a callable to generate
        kwargs based on each value in the list!
        """
        return {}

    def get_value_renderer_class(self):
        """
        Get a subclass of :class:`cradmin_legacy.viewhelpers.listbuilder.base.ItemValueRenderer`.

        Defaults to :obj:`.ViewMixin.value_renderer_class`.
        """
        return self.value_renderer_class

    def get_frame_renderer_class(self):
        """
        Get a subclass of :class:`cradmin_legacy.viewhelpers.listbuilder.base.ItemFrameRenderer`.

        Defaults to :obj:`.ViewMixin.frame_renderer_class`.
        """
        return self.frame_renderer_class

    def get_listbuilder_list_value_iterable(self, context):
        """
        Get the value_iterable for the listbuilder list.

        Must be overridden in subclasses.

        Parameters:
            context: The Django template context.
        """
        raise NotImplementedError()

    def get_listbuilder_list(self, context):
        """
        Get the listbuilder List object.

        You normally do not have to override this, but instead you should
        override:

        - :meth:`.get_listbuilder_list_value_iterable`
        - :meth:`.get_value_renderer_class`
        - :meth:`.get_frame_renderer_class`
        - :meth:`.get_value_and_frame_renderer_kwargs`
        - :meth:`.get_listbuilder_list_kwargs`

        Parameters:
            context: The Django template context.
        """
        return self.get_listbuilder_class().from_value_iterable(
            value_iterable=self.get_listbuilder_list_value_iterable(context),
            value_renderer_class=self.get_value_renderer_class(),
            frame_renderer_class=self.get_frame_renderer_class(),
            value_and_frame_renderer_kwargs=self.get_value_and_frame_renderer_kwargs(),
            **self.get_listbuilder_list_kwargs())

    def get_no_items_message(self):
        """
        Get the message to show when there are no items.

        Must be overridden in subclasses.
        """
        raise NotImplementedError()

    def add_listview_context_data(self, context):
        context['listbuilder_list'] = self.get_listbuilder_list(context)
        context['pagetitle'] = self.get_pagetitle()
        context['hide_pageheader'] = self.get_hide_page_header()
        context['pageheading'] = self.get_pageheading()
        context['no_items_message'] = self.get_no_items_message()
        context['enable_previews'] = self.get_enable_previews()
        context['pre_include_template'] = self.get_pre_include_template()
        context['buttons_include_template'] = self.get_buttons_include_template()
        context['post_include_template'] = self.get_post_include_template()

    def get_pre_include_template(self):
        """
        You can return a template to include before the listbuilder list here.
        """
        return None

    def get_buttons_include_template(self):
        """
        You can return a template to include buttons above the
        listbuilder list here. If you include this template,
        we will create a ``<p>`` with ``cradmin-legacy-listbuilderview-buttons``
        as css class, and include your template within that ``<p>``.
        """
        return None

    def get_post_include_template(self):
        """
        You can return a template to include after the listbuilder list here.
        """
        return None

    def get_context_data(self, **kwargs):
        context = super(ViewMixin, self).get_context_data(**kwargs)
        self.add_listview_context_data(context)
        return context


class ViewCreateButtonMixin(object):
    """
    Mixin class that overrides :meth:`.View.get_buttons_include_template`
    with a template that renders a create button that assumes the
    create view is named ``"create"``.
    """
    def get_buttons_include_template(self):
        return "cradmin_legacy/viewhelpers/listbuilderview/includes/create-button.django.html"


class View(ViewMixin, ListView):
    """
    View using the :doc:`viewhelpers_listbuilder`.

    Examples:

        Minimal::

            from cradmin_legacy.viewhelpers import listbuilderview

            class MyView(listbuilderview.View):
                def get_queryset(self):
                    return MyModel.objects.all()

    """

    #: The model class to list objects for. You do not have to specify
    #: this, but if you do not specify this or :meth:`~.ObjectTableView.get_model_class`,
    #: you have to override :meth:`~.ObjectTableView.get_pagetitle` and
    #: :meth:`~.ObjectTableView.get_no_items_message`.
    model = None

    #: Set this to ``True`` to make the template not render the menu.
    #: Very useful when creating foreign-key select views, and other views
    #: where you do not want your users to accidentally click out of the
    #: current view.
    hide_menu = False

    def get_model_class(self):
        """
        Get the model class to list objects for.

        Defaults to :obj:`~.View.model`. See :obj:`~.View.model` for more info.
        """
        return self.model

    def get_pagetitle(self):
        """
        Get the page title (the title tag).

        Defaults to the ``verbose_name_plural`` of the :obj:`~.View.model`
        with the first letter capitalized.
        """
        return defaultfilters.capfirst(self.get_model_class()._meta.verbose_name_plural)

    def get_listbuilder_list_value_iterable(self, context):
        return context['object_list']

    def get_queryset_for_role(self, role):
        """
        Get a queryset with all objects of :obj:`~.View.model`  that
        the current role can access.
        """
        raise NotImplementedError()

    def get_queryset(self):
        """
        DO NOT override this. Override :meth:`.get_queryset_for_role`
        instead.
        """
        queryset = self.get_queryset_for_role(self.request.cradmin_role)
        return queryset

    def get_no_items_message(self):
        """
        Get the message to show when there are no items.
        """
        return gettext_lazy('No %(modelname_plural)s') % {
            'modelname_plural': self.get_model_class()._meta.verbose_name_plural.lower(),
        }

    def disable_paging_requested(self):
        """
        If this returns ``True``, we disable paging.

        The default implementation disables paging if ``disablePaging=true`` is
        in the querystring (in ``request.GET``).

        When pagination is enabled for the listbuilder-view, disabling pagination 
        will also be supported to be able to load all instead of just the next page.
        """
        return self.request.GET.get('disablePaging', 'false') == 'true'

    def get_paginate_by(self, queryset):
        """
        Built-in support for pagination.

        Will return the set pagination (paginate_by), or disable pagination 
        if `self.disable_paging_requested` returns `True`.
        """
        if self.disable_paging_requested():
            return None
        return self.paginate_by

    def use_pagination_load_all(self):
        """
        If pagination is enabled, requesting the next page 
        will load all, not only the next page.

        This is used to tell the UI that the pagination should load 
        all elements, not just the next page setting the javasscript 
        pagination-handling mode to "loadAllOnClick".
        
        If `True`, the page-load will load all elements, and the 
        button will also show "Load all" intead of "Load more".
        """
        return False

    def get_context_data(self, **kwargs):
        context = super(View, self).get_context_data(**kwargs)
        context['cradmin_hide_menu'] = self.hide_menu
        context['pagination_mode_load_all'] = self.use_pagination_load_all()
        return context


class FilterListMixin(listfilter_viewmixin.ViewMixin):
    """
    Mixin for adding filtering with :doc:`viewhelpers.listfilter <viewhelpers_listfilter>` to
    :class:`listbuilder View <.View>`.

    Must be mixed in before any TemplateView subclass.
    """
    template_name = None

    def get_filterlist_position(self):
        """
        Get the position where you want to place the filterlist.

        Supported values are:

        - left
        - right (the default)
        - top

        Defaults to ``"top"`` if :meth:`.get_filterlist_class` returns
        :class:`cradmin_legacy.viewhelpers.listfilter.lists.Horizontal` or a subclass of it,
        otherwise ``"right"``.
        """
        filterlist_class = self.get_filterlist_class()
        if issubclass(filterlist_class, listfilter.lists.Horizontal):
            return 'top'
        else:
            return 'right'

    def get_filterlist_template_name(self):
        """
        Get the template to use based on what :meth:`.get_filterlist_position`.

        You will want to call this from the ``get_template_names`` method.
        This is just the interface, refer to the mixins implemented in
        various modules (such as :class:`cradmin_legacy.viewhelpers.listbuilderview.FilterListMixin`)
        for details on how to use this method.
        """
        if getattr(self, 'template_name', None):
            return self.template_name
        else:
            position = self.get_filterlist_position()
            template_name = 'cradmin_legacy/viewhelpers/listbuilderview/filterlist-{}.django.html'.format(position)
            return template_name

    def get_filter_unprotected_querystring_arguments(self):
        """
        This returns ``{'page'}``, which ensures we go back to
        page 1 when changing a filter.

        See :class:`cradmin_legacy.viewhelpers.listfilter.listfilter_viewmixin.ViewMixin`
        for more details.
        """
        return {'page'}

    def get_filterlist_target_dom_id(self):
        """
        Overrides
        :meth:`cradmin_legacy.viewhelpers.listfilter.listfilter_viewmixin.ViewMixin.get_filterlist_target_dom_id`
        with a default of ``"cradmin_legacy_listbuilderview_listwrapper"``.

        You should not need to override this unless you create a completely custom
        template for your view.
        """
        return 'cradmin_legacy_listbuilderview_listwrapper'



from __future__ import unicode_literals

from builtins import object

from django.conf.urls import url, include
from django.urls import reverse
from django.shortcuts import render
from django.utils.html import format_html

from cradmin_legacy.decorators import has_access_to_cradmin_instance
from . import crapp
from .registry import cradmin_instance_registry
from .views import roleselect


def reverse_cradmin_url(instanceid, appname, roleid=None,
                        viewname=crapp.INDEXVIEW_NAME,
                        args=None, kwargs=None):
    """
    Reverse an URL within a cradmin instance.

    Usage is very similar to :func:`django.urls.reverse`,
    but you specify the cradmin instance, appname, roleid and viewname
    instead of the url-name

    Examples:

        Reverse the frontpage on an app::

            myapp_index_url = reverse_cradmin_url(
                instanceid='siteadmin',
                appname='myapp',
                roleid=site.id)

        Reverse a specific view within an app::

            myapp_add_url = reverse_cradmin_url(
                instanceid='siteadmin',
                appname='myapp',
                roleid=site.id,
                viewname='add')
    """
    if roleid:
        if args:
            args = [roleid] + list(args)
        else:
            if not kwargs:
                kwargs = {}
            kwargs['roleid'] = roleid
    urlname = '{}-{}-{}'.format(instanceid, appname, viewname)
    return str(reverse(urlname, args=args, kwargs=kwargs))


class BaseCrAdminInstance(object):
    """
    Base class for a cradmin_legacy instance.

    You define a subclass of this to setup a cradmin_legacy instance.

    Attributes:
        request (HttpRequest): The current HttpRequest.
    """
    #: The ID of the cradmin instance. Must be unique for the Django
    #: instance/site. Must be a string.
    #: This is typically a short readable slug that describes what
    #: the cradmin instance does.
    #: You do not need to specify this except you need to communicate/link
    #: between cradmin instances.
    id = None

    #: The regex for matching the role id.
    #: Defaults to ``\d+``.
    roleid_regex = r'\d+'

    #: The menu class for this cradmin instance.
    menuclass = None

    #: The class defining the role for this cradmin instance.
    #: If you do not set this, the role system will not be used,
    #: which means that you will not get a value in ``request.cradmin_role``.
    roleclass = None

    #: The name of the app that the user should be redirected to
    #: after selecting a role. Subclasses MUST eighter specify
    #: this or override :meth:`rolefrontpage_url`.
    rolefrontpage_appname = None

    #: If this is ``True``, we do not prefix the urls of the
    #: :obj:`~BaseCrAdminInstance.rolefrontpage_appname` with the
    #: appname. This means that it is hosted on ``/baseurl/<roleid>/``
    #: instead of ``/baseurl/<roleid>/<appname>/``.
    #:
    #: If you couple this with setting :obj:`~.BaseCrAdminInstance.roleclass`
    #: to ``None``, the frontpage will be hosted directly on ``/baseurl/``.
    #:
    #: If you set this to ``True``, you have to be ensure that the urls of any views
    #: within the rolefrontpage app does crash any urls in any of the other apps.
    flatten_rolefrontpage_url = False

    #: Apps within the instance.
    #: Iterable of ``(appname, appclass)`` tuples where ``appname``
    #: is a slug for the app and ``appclass`` is a subclass of
    #: :class:`cradmin_legacy.crapp.App`.
    #: Can also be specified by overriding :meth:`.get_apps`.
    apps = []

    def __init__(self, request):
        """
        Parameters:
            request (HttpRequest): The current HttpRequest.
                Stored in :attr:`.request`.
        """
        self.request = request

    def get_cradmin_theme_path(self):
        """
        Return a path to a theme in the same format as :setting:`CRADMIN_LEGACY_THEME_PATH`,
        to use a custom theme for this instance.
        """
        return None

    def get_rolequeryset(self):
        """
        Get the roles for the authenticated user.

        You get the authenticated user from ``self.request.user``.
        """
        raise NotImplementedError()

    def get_titletext_for_role(self, role):
        """
        Get a short title briefly describing the given ``role``.
        """
        raise NotImplementedError()

    def get_descriptiontext_for_role(self, role):
        """
        Get a longer description for the given ``role``.

        This is never used directly on its own - it is just the default text
        used by :meth:`.get_descriptionhtml_for_role`. If you want
        HTML in your description, override :meth:`.get_descriptionhtml_for_role`
        instead.
        """
        return None

    def get_descriptionhtml_for_role(self, role):
        """
        Get a longer description for the given ``role``.
        This is always shown after/below :meth:`.get_titletext_for_role`.

        Defaults to :meth:`.get_descriptiontext_for_role` filtered to
        make it HTML safe and wrapped in a paragraph tag.
        """
        descriptiontext = self.get_descriptiontext_for_role(role)
        if descriptiontext:
            return format_html(u'<p>{}</p>', descriptiontext)
        else:
            return ''

    def get_roleid(self, role):
        """
        Get the ID for the given ``role``.
        """
        return role.pk

    def get_role_from_roleid(self, roleid):
        """
        Get the role for the given ``roleid``.

        Defaults to looking up a :obj:`roleclass` object where
        ``pk==roleid``.

        Returns:
            A role object or ``None``.
        """
        try:
            return self.roleclass.objects.get(pk=roleid)
        except self.roleclass.DoesNotExist:
            pass
        return None

    def invalid_roleid_response(self, roleid):
        """
        This is called whenever someone requests a role slug that does not exist
        (if :meth:.`get_role_from_roleid`) returns ``None``.

        Returns:
            django.http.HttpResponse: Defaults to rendering
                ``cradmin_legacy/invalid_roleid.django.html``.
        """
        return render(self.request, 'cradmin_legacy/invalid_roleid.django.html', {
            'roleid': roleid
        })

    def get_role_from_rolequeryset(self, role):
        """
        Returns the given role extracted via the :meth:`.get_rolequeryset`
        queryset.

        Raises ObjectDoesNotExist if the role is not found in the queryset.
        """
        return self.get_rolequeryset().get(pk=role.pk)

    def missing_role_response(self, role):
        """
        This is called whenever someone requests a role that exists but that
        they do not have (where meth:`.get_role_from_rolequeryset` raises ``DoesNotExist``).

        Returns:
            django.http.HttpResponse: Defaults to rendering
                ``cradmin_legacy/missing_role.django.html``
        """
        return render(self.request, 'cradmin_legacy/missing_role.django.html', {
            'role': role
        })

    def _get_menu(self):
        if not hasattr(self, '_menu'):
            menuclass = self.menuclass
            self._menu = menuclass(self)
        return self._menu

    def get_menu(self):
        """
        Get the navigation menu for the authenticated user.

        Parameters:
            user (Django User): The user requesting the menu.

        Returns:
            cradmin_legacy.crmenu.Menu: An instance of :obj:`~.BaseCrAdminInstance.menuclass` by default, but you can
            override this method to determine/create the menu dynamically.

        See Also:
            :class:`cradmin_legacy.crmenu.Menu`.
        """
        return self._get_menu()

    def reverse_url(self, appname, viewname, args=None, kwargs=None, roleid=None):
        """
        Reverse an URL within this cradmin instance.

        The advantage over using :func:`django.urls.reverse`
        is that you do not need to hardcode the id of the cradmin instance,
        and that the ``roleid`` is automatically added to args or kwargs
        (depending on which one you use to pass arguments to the url).

        Args:
            appname (str): The name of the app.
            viewname (str): The name of the view within the app.
            args (list): Args for the view
            kwargs (dict): Keyword args for the view.
            roleid: The roleid.
        """
        kwargs = {
            'instanceid': self.id,
            'appname': appname,
            'viewname': viewname,
            'args': args,
            'kwargs': kwargs
        }
        if self.roleclass:
            if roleid is None:
                roleid = self.get_roleid(self.request.cradmin_role)
            kwargs['roleid'] = roleid
        return reverse_cradmin_url(**kwargs)

    def appindex_url(self, appname, args=None, kwargs=None, roleid=None):
        """
        Reverse the url of the landing page for the given app.

        The landing page is the view named :obj:`cradmin_legacy.crapp.INDEXVIEW_NAME`.

        This would be the same as using :meth:`.reverse_url` with ``viewname=crapp.INDEXVIEW_NAME``.

        Args:
            appname (str): The name of the app.
            args (list): Args for the view
            kwargs (dict): Keyword args for the view.
            roleid: The roleid.
        """
        return str(self.reverse_url(appname, viewname=crapp.INDEXVIEW_NAME, roleid=roleid,
                                args=args, kwargs=kwargs))

    def rolefrontpage_url(self, roleid=None):
        """
        Returns the URL that the user should be redirected to
        after selecting a role.
        """
        if self.roleclass:
            if roleid is None:
                roleid = self.get_roleid(self.request.cradmin_role)
        return self.appindex_url(self.rolefrontpage_appname, roleid=roleid)

    def get_instance_frontpage_url(self):
        """
        Return the URL of the instance frontpage view.

        See:
            :meth:`.get_instance_frontpage_url`.
        """
        if self.__class__.__no_role_and_flatten_rolefrontpage_url():
            return self.rolefrontpage_url()
        else:
            return reverse('{}-frontpage'.format(self.id))

    def roleselectview_url(self):
        """
        Deprecated, use :meth:`.get_instance_frontpage_url` instead.
        """
        return self.get_instance_frontpage_url()

    def get_common_http_headers(self):
        """
        Override this to set common HTTP headers for all views in the instance.

        Returns:
            A mapping object mapping HTTP header name to value. Returns
            empty dict by default.
        """
        return {}

    def has_access(self):
        """
        Check if the given user has access to this cradmin instance.

        Defaults to ``self.request.user.is_authenticated``, but you
        can override this.
        """
        return self.request.user.is_authenticated

    def get_foreignkeyselectview_url(self, model_class):
        """
        Get foreign key select view URL for the given model class.

        This can be used by foreign key select widgets to lookup a view
        for this model within the current instance.

        By default this returns ``None``, so you have to override this
        if you want to use it.

        Parameters:
            model_class: A ``django.db.models.Model`` subclass.
        """
        return None

    def get_manytomanyselectview_url(self, model_class):
        """
        Get many-to-many select view URL for the given model class.

        This can be used by many-to-many widgets,
        like :class:`cradmin_legacy.widgets.modelmultichoice.ModelMultiChoiceWidget`,
        to lookup a view for this model within the current instance.

        By default this returns ``None``, so you have to override this
        if you want to use it.

        Parameters:
            model_class: A ``django.db.models.Model`` subclass.
        """
        return None

    @classmethod
    def get_roleselect_view(cls):
        """
        The view for selecting role.

        Defaults to::

            from cradmin_legacy.views import roleselect
            from django.contrib.auth.decorators import login_required
            return login_required(roleselect.RoleSelectView.as_view())

        If you want to provide your own role select view, you can simply implement
        it here. Another option is to extend RoleSelectView and override the ``template_name``.

        Note:

            The name of the URL for this view is ``<cradmin instance id>-roleselect``,
            where ``<cradmin instance id>`` is :obj:`.id`. You can reverse the URL of
            this view with :meth:`.get_instance_frontpage_url`.
        """
        return has_access_to_cradmin_instance(roleselect.RoleSelectView.as_view())

    @classmethod
    def get_instance_frontpage_view(cls):
        if cls.roleclass:
            return cls.get_roleselect_view()
        else:
            raise NotImplementedError('When you do not define a roleclass, you have '
                                      'to override get_instance_frontpage_view()')

    @classmethod
    def matches_urlpath(cls, urlpath):
        """
        If you have more than one BaseCrAdminInstance in a single Django
        instance, you have to implement this classmethod on each of the
        instances to be able to lookup/detect the _current_ BaseCrAdminInstance.

        Parameters:
            urlpath: The url path (HttpRequest.path) to match against.

        Returns:
            ``True`` if the given urlpath is for a view within this CrAdmin instance.
        """
        raise NotImplementedError()

    @classmethod
    def get_apps(cls):
        """
        See :obj:`.apps`. Defaults to returning :obj:`.apps`, but can be overridden.
        """
        return cls.apps

    @classmethod
    def get_app_url(cls, appname, appclass):
        appurlpatterns = appclass.build_urls(cls.id, appname)
        flatten = cls.rolefrontpage_appname == appname and cls.flatten_rolefrontpage_url
        if cls.roleclass:
            if flatten:
                return url(r'^(?P<roleid>{})/'.format(cls.roleid_regex),
                           include(appurlpatterns))
            else:
                return url(r'^(?P<roleid>{})/{}/'.format(cls.roleid_regex, appname),
                           include(appurlpatterns))

        else:
            if flatten:
                return url(r'^', include(appurlpatterns))
            else:
                return url(r'^{}/'.format(appname),
                           include(appurlpatterns))

    @classmethod
    def _get_app_urls(cls):
        urls = []
        for appname, appclass in cls.get_apps():
            urls.append(cls.get_app_url(appname=appname, appclass=appclass))
        return urls

    @classmethod
    def __no_role_and_flatten_rolefrontpage_url(cls):
        return cls.flatten_rolefrontpage_url and not cls.roleclass

    @classmethod
    def urls(cls):
        """
        Get the url patterns for the cradmin instance.
        """
        cradmin_instance_registry.add(cls)
        urls = cls._get_app_urls()
        if not cls.__no_role_and_flatten_rolefrontpage_url():
            urls.append(url('^$', cls.get_instance_frontpage_view(),
                            name='{}-frontpage'.format(cls.id)))

        return urls

    def add_extra_instance_variables_to_request(self, request):
        """
        Override this method to add extra attributes to the request object
        for all views in this cradmin instance.

        This is called by the decorator that wraps all views within the instance.
        """
        pass



from __future__ import unicode_literals

from django.views.generic import TemplateView

from cradmin_legacy.apps.cradmin_resetpassword.views.begin import PasswordResetEmail


class EmailSentView(TemplateView):
    template_name = 'cradmin_resetpassword/email_sent.django.html'

    def get_context_data(self, **kwargs):
        context = super(EmailSentView, self).get_context_data(**kwargs)
        context['email_subject'] = PasswordResetEmail().render_subject()
        return context



from __future__ import unicode_literals

from builtins import object
from functools import update_wrapper

from django.conf.urls import url

from .decorators import cradminview, has_access_to_cradmin_instance

#: The name of the app index view (the landing page for the app).
#: We do not enforce this, but we assume that each app has a
#: view with this name.
INDEXVIEW_NAME = 'INDEX'


class Url(object):
    """
    Url is mostly the same as func:`django.conf.urls.url`.
    You use Url to add urls to an app.
    """
    def __init__(self, regex, view, kwargs=None, name=None):
        """
        Args:
            regex: The URL regex.
            view: The view (E.g.: ``MyView.as_view()``).
            kwargs: Keyword arguments for the view.
            name: The name of the view. This just have to be unique within
                the :class:`.App` - the actual URL name is generated
                based on the app name and the :obj:`cradmin_legacy.crinstance.BaseCrAdminInstance.id`.
        """
        self.regex = regex
        self.view = view
        self.kwargs = kwargs
        self.name = name


class App(object):
    """
    A cradmin App.

    Added to a :class:`cradmin_legacy.crinstance.BaseCrAdminInstance`
    with :obj:`cradmin_legacy.crinstance.BaseCrAdminInstance.apps`.
    """
    #: See :meth:`~.App.get_appurls`.
    appurls = []

    def __init__(self, appname, request, active_viewname):
        self.appname = appname
        self.request = request
        self.active_viewname = active_viewname

    def reverse_appurl(self, viewname, args=None, kwargs=None):
        """
        Works just like :func:`django.urls.reverse`, except
        that the name is the name given in :obj:`.appurls`, not the full
        name of the URL.

        This means that you should use this to reverse urls within this app.
        """
        return self.request.cradmin_instance.reverse_url(
            appname=self.appname, viewname=viewname,
            args=args, kwargs=kwargs)

    def reverse_appindexurl(self, args=None, kwargs=None):
        """
        Shortcut for::

            reverse_appurl(crapp.INDEXVIEW_NAME, args=args, kwargs=kwargs)
        """
        return self.reverse_appurl(viewname=INDEXVIEW_NAME, args=args, kwargs=kwargs)

    @classmethod
    def get_appurls(cls):
        """
        Get app URLs. Defaults to :obj:`~.App.appurls`.

        Returns:
            A list of :class:`.Url` objects.
        """
        return cls.appurls

    @classmethod
    def _wrap_view(cls, appname, view, viewname):
        def viewwrapper(request, *args, **kwargs):
            request.cradmin_app = cls(appname, request, active_viewname=viewname)
            return has_access_to_cradmin_instance(cradminview(view))(request, *args, **kwargs)

        # Take name and docstring from view
        update_wrapper(viewwrapper, view, updated=())

        return viewwrapper

    @classmethod
    def build_urls(cls, cradmin_instance_id, appname):
        """
        Used internally by :meth:`cradmin_legacy.crinstance.BaseCrAdminInstance.urls`
        to build urls for all views in the app.
        """
        urls = []
        for pattern in cls.get_appurls():
            urls.append(
                url(
                    pattern.regex, cls._wrap_view(appname, pattern.view, pattern.name),
                    name='{}-{}-{}'.format(cradmin_instance_id, appname, pattern.name),
                    kwargs=pattern.kwargs))
        return urls



from __future__ import unicode_literals
from cradmin_legacy.renderable import AbstractRenderableWithCss


class AbstractItemRenderer(AbstractRenderableWithCss):
    """
    Abstract base class for :class:`.ItemFrameRenderer`
    and :class:`.ItemValueRenderer`.

    You will normally add objects of a class that extends this class
    to :class:`.List`, but it is not required.
    """

    #: If this is specified, we will add an attribute with this name
    #: for the value as an attribute of the object.
    #:
    #: I.e.: if ``valuealias = "person"``, you will be able to use ``me.person`` in
    #: the template, and you will be able to use ``self.person`` in any methods you
    #: add or override in the class (just remember to call ``super()`` if you override
    #: ``__init__``).
    valuealias = None

    def __init__(self, value, **kwargs):
        """
        Parameters:
            value: The value to render.
        """
        self.value = value
        self.kwargs = kwargs
        if self.valuealias:
            setattr(self, self.valuealias, self.value)


class ItemValueRenderer(AbstractItemRenderer):
    """
    The value renderer renders the value of each item in
    the :class:`.List`.
    """
    template_name = 'cradmin_legacy/viewhelpers/listbuilder/base/itemvalue.django.html'

    def get_base_css_classes_list(self):
        """
        Override this to set your own css classes.
        """
        return ['cradmin-legacy-listbuilder-itemvalue']


class ItemFrameRenderer(AbstractItemRenderer):
    """
    The outer item renderer provides a frame around the item.

    This is typically subclassed to:

    - Add visually highlighted frames for some items.
    - Add checkbox for each item
    - Make the item clickable (via an ``a`` tag or javascript).

    .. note::

        The :class:`.List` dors not require a :class:`.ItemFrameRenderer`,
        so lightweight lists without this extra wrapper is possible.

    .. attribute:: inneritem

        The renderable this frame wraps - an object of
        :class:`.ItemValueRenderer` or a subclass.
    """
    template_name = 'cradmin_legacy/viewhelpers/listbuilder/base/itemframe.django.html'

    def __init__(self, inneritem, **kwargs):
        super(ItemFrameRenderer, self).__init__(inneritem.value, **kwargs)
        self.inneritem = inneritem

    def get_base_css_classes_list(self):
        """
        Override this to set your own css classes.
        """
        return ['cradmin-legacy-listbuilder-itemframe']


class List(AbstractRenderableWithCss):
    """
    A builder for HTML lists (can be used for other lists as well).

    Each item in the list is a :class:`.AbstractItemRenderer` object, and
    it is typically:

    - A subclass of :class:`.ItemValueRenderer` for simple lists.
    - A subclass of :class:`.ItemFrameRenderer` for more complex lists.

    .. note:: Items can be any :class:`cradmin_legacy.renderable.AbstractRenderableWithCss`, but that does not
        work with the shortcut methods:

        - :meth:`.extend_with_values`
        - :meth:`.from_value_iterable`
    """
    template_name = 'cradmin_legacy/viewhelpers/listbuilder/base/list.django.html'

    def __init__(self):
        self.renderable_list = []

    def iter_renderables(self):
        """
        Get an iterator over the items in the list.
        """
        return iter(self.renderable_list)

    def has_items(self):
        """
        Returns:
            bool: ``True`` if the list has any items, and ``False`` otherwise.
        """
        return len(self.renderable_list) > 0

    def get_base_css_classes_list(self):
        """
        Sets ``cradmin-legacy-listbuilder-list`` as css class for the list.
        """
        return ['cradmin-legacy-listbuilder-list']

    def append(self, renderable):
        """
        Appends an item to the list.

        Parameters:
            renderable: Must implement the :class:`cradmin_legacy.renderable.AbstractRenderableWithCss` interface,
                and it is typically a subclass of :class:`.AbstractItemRenderer`.
        """
        self.renderable_list.append(renderable)

    def extend(self, renerable_iterable):
        """
        Just like :meth:`.append` except that it takes an iterable
        of renderables instead of a single renderable.
        """
        self.renderable_list.extend(renerable_iterable)

    def get_default_value_renderer_class(self):
        """
        The default ``value_renderer_class`` for :meth:`.extend_with_values`.
        """
        return ItemValueRenderer

    def get_default_frame_renderer_class(self):
        """
        The default ``frame_renderer_class`` for :meth:`.extend_with_values`.
        """
        return None

    def extend_with_values(self, value_iterable,
                           value_renderer_class=None,
                           frame_renderer_class=None,
                           value_and_frame_renderer_kwargs=None):
        """
        Extends the list with an iterable of values.

        Parameters:
            value_iterable: An iterable of values to add to the list.
            value_renderer_class: A subclass of :class:`.ItemValueRenderer`.
                Defaults to :meth:`.get_default_value_renderer_class`.
            frame_renderer_class: A subclass of :class:`.ItemFrameRenderer`.

                If this is ``None``, the object added to the list will be an object of the
                ``value_renderer_class``.

                If this is specified, the object added to the list will be an object
                of the ``frame_renderer_class`` with an object of the ``value_renderer_class``
                as the ``inneritem`` attribute/parameter.

                Defaults to :meth:`.get_default_frame_renderer_class`.
            value_and_frame_renderer_kwargs (dict or function): Keyword arguments to send to all
                the constructor of all the value and frame renderers.

                If this is not a dict, it is assumed to be a function/callable that takes
                a value from the ``value_iterable`` as the only argument, and returns
                a dict of kwargs. Example function::

                    def my_kwargs_generator(value):
                        if value == 10:
                            return {
                                'is_selected': True
                            }
                        else:
                            return {}

                The ``value``-argument is sent as a positional argument, so your
                function can name the input argument as you like (so if the value
                is a ``Product`` object, you can use ``def my_kwargs_generator(product): ...``.
        """
        value_renderer_class = value_renderer_class or self.get_default_value_renderer_class()
        frame_renderer_class = frame_renderer_class or self.get_default_frame_renderer_class()
        value_and_frame_renderer_kwargs = value_and_frame_renderer_kwargs or {}
        for value in value_iterable:
            if isinstance(value_and_frame_renderer_kwargs, dict):
                kwargs = value_and_frame_renderer_kwargs
            else:
                kwargs = value_and_frame_renderer_kwargs(value)
            value_renderable = value_renderer_class(value=value, **kwargs)
            if frame_renderer_class:
                renderable = frame_renderer_class(inneritem=value_renderable, **kwargs)
            else:
                renderable = value_renderable
            self.append(renderable)

    @classmethod
    def from_value_iterable(cls, value_iterable,
                            value_renderer_class=None,
                            frame_renderer_class=None,
                            value_and_frame_renderer_kwargs=None,
                            **listkwargs):
        """
        A shortcut for creating an object of this class with the given ``**listkwargs``
        as __init__ arguments, and then calling :meth:`.extend_with_values` with
        ``value_iterable``, ``value_renderer_class`` and ``frame_renderer_class`` as
        arguments.

        See :meth:`.extend_with_values` for documentation for the arguments.

        Returns:
            An object of this class.
        """
        listobject = cls(**listkwargs)
        listobject.extend_with_values(value_iterable=value_iterable,
                                      value_renderer_class=value_renderer_class,
                                      frame_renderer_class=frame_renderer_class,
                                      value_and_frame_renderer_kwargs=value_and_frame_renderer_kwargs)
        return listobject



from __future__ import unicode_literals

from django.urls import path

from cradmin_legacy.apps.cradmin_activate_account.views.activate import ActivateAccountView

urlpatterns = [
    path('activate/<str:token>',
        ActivateAccountView.as_view(),
        name="cradmin-activate-account-activate"),
]



from __future__ import unicode_literals

from django.urls import path
from django.contrib.auth.decorators import login_required

from cradmin_legacy.apps.cradmin_temporaryfileuploadstore.views.temporary_file_upload_api import \
    UploadTemporaryFilesView

urlpatterns = [
    path('temporary_file_upload_api',
        login_required(UploadTemporaryFilesView.as_view()),
        name='cradmin_temporary_file_upload_api'),
]



from __future__ import unicode_literals
from builtins import str


def human_readable_filesize(size_in_bytes):
    """
    Humanize the given file size in bytes.

    Returns a number suffixed with ``B``, ``KB``, ``MB``,
    ``GB`` or ``TB``.

    Examples:

        >>> from cradmin_legacy.utils import crhumanize
        >>> crhumanize.human_readable_filesize(1)
        '1B'
        >>> crhumanize.human_readable_filesize(2344234345)
        '2.34GB'
        >>> crhumanize.human_readable_filesize(23442343451234)
        '23.44TB'
    """
    if size_in_bytes < 1000:
        return '{}B'.format(size_in_bytes)
    if size_in_bytes < 1000000:
        return '{}KB'.format(int(size_in_bytes / 1000.0))
    elif size_in_bytes < 1000000000:
        return '{:.1f}MB'.format(size_in_bytes / 1000000.0)
    elif size_in_bytes < 1000000000000:
        return '{:.2f}GB'.format(size_in_bytes / 1000000000.0)
    else:
        return '{:.2f}TB'.format(size_in_bytes / 1000000000000.0)


def dehumanize_readable_filesize(humanized_size):
    """
    Does the opposite of :func:`.human_readable_filesize`.

    Takes a string containing a number suffixed with ``B``, ``KB``, ``MB``,
    ``GB`` or ``TB``, and returns an int with the number of bytes.

    Examples:

        >>> from cradmin_legacy.utils import crhumanize
        >>> crhumanize.dehumanize_readable_filesize('999B')
        999
        >>> crhumanize.dehumanize_readable_filesize('2.34GB')
        2340000000
        >>> crhumanize.dehumanize_readable_filesize('43.312TB')
        43312000000000
    """
    if not isinstance(humanized_size, str):
        raise ValueError('humanized_size must be a string (unicode in python 2)')
    humanized_size = humanized_size[:-1]
    if humanized_size[-1].isdigit():
        return int(humanized_size)
    else:
        sizechar = humanized_size[-1]
        number = float(humanized_size[:-1])
        multiply_by = {
            'K': 1000,
            'M': 1000000,
            'G': 1000000000,
            'T': 1000000000000
        }[sizechar]
        return int(number * multiply_by)



from __future__ import unicode_literals

from django.urls import path

from cradmin_legacy.apps.cradmin_resetpassword.views.begin import BeginPasswordResetView
from cradmin_legacy.apps.cradmin_resetpassword.views.email_sent import EmailSentView
from cradmin_legacy.apps.cradmin_resetpassword.views.reset import ResetPasswordView

urlpatterns = [
    path('begin',
         BeginPasswordResetView.as_view(),
         name="cradmin-resetpassword-begin"),
    path('email-sent',
         EmailSentView.as_view(),
         name="cradmin-resetpassword-email-sent"),
    path('reset/<str:token>',
         ResetPasswordView.as_view(),
         name="cradmin-resetpassword-reset"),
]



from django.utils.translation import gettext_lazy, pgettext, pgettext_lazy

from cradmin_legacy.viewhelpers.listfilter.base.abstractfilter import AbstractFilter


class AbstractInputFilter(AbstractFilter):
    """
    Abstract base class for any filter that uses a single text input field.
    """
    template_name = 'cradmin_legacy/viewhelpers/listfilter/django/single/textinput/base.django.html'

    #: The text used in URLs as placeholder for the value of this filter.
    #: The AngularJS directive replaces this with the actual value.
    #: You should not have to override this.
    urlpattern_replace_text = '_-_TEXTINPUT_-_VALUE_-_'

    def get_input_html_element_type(self):
        """
        Get the ``type``-attribute of the input HTML element.

        Defaults to ``"text"``.
        """
        return 'text'

    def get_input_html_element_pattern(self):
        """
        Get the ``pattern`` attribute of the input HTML element.

        Defaults to ``None``. If this returns a value that
        is ``bool(value) == False``, the pattern attribute is
        not added to the input HTML element.
        """
        return None

    def get_base_css_classes_list(self):
        css_classes = super(AbstractInputFilter, self).get_base_css_classes_list()
        css_classes.append('cradmin-legacy-listfilter-filter-single-input')
        return css_classes

    def get_placeholder(self):
        """
        Get the placeholder text for the input element.

        Defaults to empty string.
        """
        return ''

    def get_inputfield_value(self):
        return self.get_cleaned_value() or ''

    def get_urlpattern(self):
        """
        Get the URL pattern to use when the input value is a non-empty string.

        You should not need to override this.
        """
        return self.build_set_values_url(values=[self.urlpattern_replace_text])

    def get_emptyvalue_url(self):
        """
        Get the URL to use when the input value is an empty string.

        You should not need to override this.
        """
        return self.build_clear_values_url()

    def get_timeout_milliseconds(self):
        """
        Get the number of milliseconds to wait until applying the change
        when users modify the input field.

        When users hit enter or focus away from the field, search is
        done instantly, so this is only the delay to wait while
        users are typing (live search).
        """
        return 500

    def get_angularjs_options_dict(self):
        options_dict = super(AbstractInputFilter, self).get_angularjs_options_dict()
        options_dict['timeout_milliseconds'] = self.get_timeout_milliseconds()
        options_dict['urlpattern_replace_text'] = self.urlpattern_replace_text
        return options_dict


class IntInputFilterMixin(object):
    """
    Mixin class that can be used with a subclass of
    :class:`.AbstractInputFilter` to provide int input
    instead of text.

    Must be mixed in **before** :class:`AbstractInputFilter`.
    """
    def get_cleaned_value(self):
        """
        Get the value as an ``int``, or ``None`` (if no value is provided).
        """
        cleaned_value = super(IntInputFilterMixin, self).get_cleaned_value()
        if cleaned_value in (None, ''):
            return None
        try:
            return int(cleaned_value)
        except ValueError:
            return None

    def get_placeholder(self):
        return pgettext_lazy('listfilter IntInputFilterMixin', 'Type a number ...')


class AbstractSearch(AbstractInputFilter):
    """
    Abstract search filter.

    Subclasses only need to implement
    :meth:`cradmin_legacy.viewhelpers.listfilter.base.abstractfilter.AbstractFilter.filter`.

    See :class:`cradmin_legacy.viewhelpers.listfilter.django.single.textinput.Search`
    for a Django ORM implementation.
    """
    def get_base_css_classes_list(self):
        css_classes = super(AbstractSearch, self).get_base_css_classes_list()
        css_classes.append('cradmin-legacy-listfilter-filter-single-input-search')
        return css_classes

    def get_placeholder(self):
        return gettext_lazy('Search ...')

    def get_loadingmessage(self):
        return pgettext('listfilter loading message', 'Searching')



from __future__ import unicode_literals
from django.test import TestCase
from django.urls import reverse
import htmls
from cradmin_legacy.apps.cradmin_authenticate.tests.cradmin_authenticate_testapp.models import EmailUser
from django.test.utils import override_settings

from cradmin_legacy.tests.views.helpers import create_testuser


class TestUsernameLogin(TestCase):
    def setUp(self):
        self.testuser = create_testuser(username='testuser')
        self.url = reverse('cradmin-authenticate-login')

    def test_get(self):
        response = self.client.get(self.url)
        self.assertEqual(response.status_code, 200)
        selector = htmls.S(response.content)
        self.assertEqual(selector.one('h1').alltext_normalized, 'Sign in')

    def test_login_ok(self):
        response = self.client.post(self.url, {
            'username': 'testuser',
            'password': 'test'
        })
        self.assertEqual(response.status_code, 302)

    def test_login_invalid(self):
        response = self.client.post(self.url, {
            'username': 'testuser',
            'password': 'invalid'
        })
        self.assertEqual(response.status_code, 200)
        selector = htmls.S(response.content)
        self.assertIn(
            "Your username and password didn't match",
            selector.one('#cradmin_authenticate_login_form').alltext_normalized)


class TestEmailLogin(TestCase):
    def setUp(self):
        self.testuser = EmailUser(email='testuser@example.com')
        self.testuser.set_password('test')
        self.testuser.save()
        self.url = reverse('cradmin-authenticate-login')

    def test_get(self):
        response = self.client.get(self.url)
        self.assertEqual(response.status_code, 200)
        selector = htmls.S(response.content)
        self.assertEqual(selector.one('h1').alltext_normalized, 'Sign in')

    def test_login_ok(self):
        with self.settings(LOGIN_REDIRECT_URL='/login/redirect'):
            response = self.client.post(self.url, {
                'email': 'testuser@example.com',
                'password': 'test'
            })
        self.assertEqual(response.status_code, 302)
        self.assertEqual(response['location'], '/login/redirect')

    def test_login_next(self):
        response = self.client.post('{}?next=/next'.format(self.url), {
            'email': 'testuser@example.com',
            'password': 'test'
        })
        self.assertEqual(response.status_code, 302)
        self.assertEqual(response['location'], '/next')

    def test_login_invalid(self):
        response = self.client.post(self.url, {
            'email': 'testuser@example.com',
            'password': 'invalid'
        })
        self.assertEqual(response.status_code, 200)
        selector = htmls.S(response.content)
        self.assertIn(
            "Your email and password didn't match",
            selector.one('#cradmin_authenticate_login_form').alltext_normalized)

TestEmailLogin = override_settings(
    AUTH_USER_MODEL='cradmin_authenticate_testapp.EmailUser',
    CRADMIN_LEGACY_USE_EMAIL_AUTH=True
)(TestEmailLogin)
